Package: grid
Version: 3.5.1
Priority: base
Title: The Grid Graphics Package
Author: Paul Murrell <paul@stat.auckland.ac.nz>
Maintainer: R Core Team <R-core@r-project.org>
Description: A rewrite of the graphics layout capabilities, plus some
  support for interaction.
Imports: grDevices, utils
Suggests: lattice
License: Part of R 3.5.1
NeedsCompilation: yes
Built: R 3.5.1; armv6l-unknown-linux-gnueabihf; 2018-10-30 04:44:03 UTC; unix
