Package: grid
Version: 3.3.1
Priority: base
Title: The Grid Graphics Package
Author: Paul Murrell <paul@stat.auckland.ac.nz>
Maintainer: R Core Team <R-core@r-project.org>
Description: A rewrite of the graphics layout capabilities, plus some
  support for interaction.
Imports: grDevices, utils
Suggests: lattice
License: Part of R 3.3.1
Built: R 3.3.1; aarch64-unknown-linux-gnu; 2016-06-25 00:19:20 UTC; unix
