Package: graphics
Version: 3.1.2
Priority: base
Title: The R Graphics Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: R functions for base graphics
Imports: grDevices
License: Part of R 3.1.2
Built: R 3.1.2; x86_64-unknown-cygwin; 2015-01-30 23:11:31 UTC; unix
