Package: grDevices
Version: 3.4.0
Priority: base
Title: The R Graphics Devices and Support for Colours and Fonts
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: Graphics devices and support for base and grid graphics.
License: Part of R 3.4.0
Suggests: KernSmooth
NeedsCompilation: yes
Built: R 3.4.0; armv7l-unknown-linux-gnueabihf; 2017-04-29 18:31:59 UTC; unix
