Package: tools
Version: 3.2.0
Priority: base
Title: Tools for Package Development
Author: R Core Team
Maintainer: R Core Team <R-core@r-project.org>
Description: Tools for package development, administration and documentation.
License: Part of R 3.2.0
Suggests: XML
Built: R 3.2.0; armv5tel-unknown-linux-gnueabi; 2015-06-15 08:21:11 UTC; unix
