Package: tools
Version: 3.2.3
Priority: base
Title: Tools for Package Development
Author: R Core Team
Maintainer: R Core Team <R-core@r-project.org>
Description: Tools for package development, administration and documentation.
License: Part of R 3.2.3
Suggests: XML, codetools, methods
Built: R 3.2.3; armv7l-unknown-linux-gnueabihf; 2015-12-12 12:26:38 UTC; unix
