Package: tools
Version: 3.4.0
Priority: base
Title: Tools for Package Development
Author: R Core Team
Maintainer: R Core Team <R-core@r-project.org>
Description: Tools for package development, administration and documentation.
License: Part of R 3.4.0
Suggests: codetools, methods, xml2, curl
NeedsCompilation: yes
Built: R 3.4.0; armv7l-unknown-linux-gnueabihf; 2017-04-29 18:29:25 UTC; unix
