Package: tools
Version: 3.3.3
Priority: base
Title: Tools for Package Development
Author: R Core Team
Maintainer: R Core Team <R-core@r-project.org>
Description: Tools for package development, administration and documentation.
License: Part of R 3.3.3
Suggests: codetools, methods, xml2
NeedsCompilation: yes
Built: R 3.3.3; aarch64-unknown-linux-gnu; 2017-03-06 12:53:06 UTC; unix
