Package: tools
Version: 3.4.3
Priority: base
Title: Tools for Package Development
Author: R Core Team
Maintainer: R Core Team <R-core@r-project.org>
Description: Tools for package development, administration and documentation.
License: Part of R 3.4.3
Suggests: codetools, methods, xml2, curl
NeedsCompilation: yes
Built: R 3.4.3; armv5tel-unknown-linux-gnueabi; 2017-11-30 19:47:43 UTC; unix
