Package: grDevices
Version: 4.0.3
Priority: base
Title: The R Graphics Devices and Support for Colours and Fonts
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: Graphics devices and support for base and grid graphics.
License: Part of R 4.0.3
Suggests: KernSmooth
NeedsCompilation: yes
Built: R 4.0.3; armv5tel-unknown-linux-gnueabi; 2020-12-23 05:20:51 UTC; unix
