Package: grDevices
Version: 4.0.2
Priority: base
Title: The R Graphics Devices and Support for Colours and Fonts
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: Graphics devices and support for base and grid graphics.
License: Part of R 4.0.2
Suggests: KernSmooth
NeedsCompilation: yes
Built: R 4.0.2; aarch64-unknown-linux-gnu; 2020-06-22 09:21:21 UTC; unix
