Package: grDevices
Version: 4.2.0
Priority: base
Title: The R Graphics Devices and Support for Colours and Fonts
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: Graphics devices and support for base and grid graphics.
License: Part of R 4.2.0
Suggests: KernSmooth
NeedsCompilation: yes
Built: R 4.2.0; armv7l-unknown-linux-gnueabihf; 2022-04-22 13:26:08 UTC; unix
