Package: tools
Version: 4.0.0
Priority: base
Title: Tools for Package Development
Author: R Core Team
Maintainer: R Core Team <R-core@r-project.org>
Description: Tools for package development, administration and documentation.
License: Part of R 4.0.0
Suggests: codetools, methods, xml2, curl, commonmark
NeedsCompilation: yes
Built: R 4.0.0; aarch64-unknown-linux-gnu; 2020-04-24 12:46:04 UTC; unix
