Package: tools
Version: 4.0.3
Priority: base
Title: Tools for Package Development
Author: R Core Team
Maintainer: R Core Team <R-core@r-project.org>
Description: Tools for package development, administration and documentation.
License: Part of R 4.0.3
Suggests: codetools, methods, xml2, curl, commonmark
NeedsCompilation: yes
Built: R 4.0.3; armv5tel-unknown-linux-gnueabi; 2020-12-23 05:17:45 UTC; unix
