Package: stats4
Title: Statistical Functions using S4 Classes
Version: 4.0.3
Priority: base
Author: R Core Team and contributors worldwide
Description: Statistical Functions using S4 classes.
Maintainer: R Core Team <R-core@r-project.org>
Imports: graphics, methods, stats
License: Part of R 4.0.3
Built: R 4.0.3; ; 2020-10-10 16:11:40 UTC; unix
