Title: Survival Analysis
Maintainer: Terry M Therneau <therneau.terry@mayo.edu>
Priority: recommended
Package: survival
Version: 2.37-7
Depends: stats, utils, graphics, splines, R (>= 2.13.0)
LazyData: Yes
LazyLoad: Yes
ByteCompile: Yes
Authors@R: c(person(c("Terry", "M"), "Therneau",
                    email="therneau.terry@mayo.edu",
	            role=c("aut", "cre")),
             person("Thomas", "Lumley", role=c("ctb", "trl"),
	       comment="original S->R port and maintainer until 2009"))
Description: survival analysis: descriptive statistics, two-sample
  tests, parametric accelerated failure models, Cox model. Delayed entry
  (truncation) allowed for all models; interval censoring for parametric
  models. Case-cohort designs.
License: LGPL (>= 2)
URL: http://r-forge.r-project.org
Packaged: 2014-01-22 13:49:28 UTC; therneau
Author: Terry M Therneau [aut, cre],
  Thomas Lumley [ctb, trl] (original S->R port and maintainer until 2009)
NeedsCompilation: yes
Repository: CRAN
Date/Publication: 2014-01-22 17:48:51
Built: R 3.1.2; i686-pc-cygwin; 2015-01-31 00:02:33 UTC; unix
