Package: Matrix
Version: 1.1-4
Date: 2014-06-14
Priority: recommended
Title: Sparse and Dense Matrix Classes and Methods
Author: Douglas Bates <bates@stat.wisc.edu> and Martin Maechler
Maintainer: Martin Maechler <mmaechler+Matrix@gmail.com>
Contact: Doug and Martin <Matrix-authors@R-project.org>
Description: Classes and methods for dense and sparse matrices and
    operations on them using LAPACK and SuiteSparse.
Depends: R (>= 2.15.2), methods
Imports: graphics, grid, stats, utils, lattice
Suggests: expm, MASS
Enhances: MatrixModels, graph, SparseM, sfsmisc
Encoding: UTF-8
LazyData: no
LazyDataNote: not possible, since we use data/*.R *and* our classes
ByteCompile: yes
BuildResaveData: no
License: GPL (>= 2)
LicenseNote: The Matrix package includes libraries AMD, CHOLMOD,
        COLAMD, CSparse and SPQR from the SuiteSparse collection of Tim
        Davis.  All sections of that code are covered by the GPL or
        LGPL licenses.  See the directory doc/UFsparse for details.
URL: http://Matrix.R-forge.R-project.org/
Packaged: 2014-06-14 16:22:31 UTC; maechler
NeedsCompilation: yes
Repository: CRAN
Date/Publication: 2014-06-15 20:25:32
Built: R 3.1.2; i686-pc-cygwin; 2015-01-30 23:59:20 UTC; unix
