Package: compiler
Version: 3.0.2
Priority: base
Title: The R Compiler Package
Author: Luke Tierney <luke-tierney@uiowa.edu>
Maintainer: R Core Team <R-core@r-project.org>
Description: Byte code compiler for R
License: Part of R 3.0.2
Built: R 3.0.2; ; 2018-05-02 07:57:03 UTC; unix
