Package: grid
Version: 3.1.2
Priority: base
Title: The Grid Graphics Package
Author: Paul Murrell <paul@stat.auckland.ac.nz>
Maintainer: R Core Team <R-core@r-project.org>
Description: A rewrite of the graphics layout capabilities, plus some
  support for interaction
Imports: grDevices
Suggests: lattice
License: Part of R 3.1.2
Built: R 3.1.2; i686-pc-cygwin; 2015-01-30 23:55:50 UTC; unix
