Package: graphics
Version: 4.4.0
Priority: base
Title: The R Graphics Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: R functions for base graphics.
Imports: grDevices
License: Part of R 4.4.0
NeedsCompilation: yes
Enhances: vcd
Built: R 4.4.0; i386-portbld-midnightbsd4.0; 2026-05-11 16:53:26 UTC; unix
