Package: grid
Version: 4.4.3
Priority: base
Title: The Grid Graphics Package
Author: Paul Murrell <paul@stat.auckland.ac.nz>
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: A rewrite of the graphics layout capabilities, plus some
  support for interaction.
Imports: grDevices, utils
License: Part of R 4.4.3
NeedsCompilation: yes
Built: R 4.4.3; armv7l-unknown-linux-gnueabihf; 2025-02-28 16:31:22 UTC; unix
