Package: tcltk
Version: 3.2.0
Priority: base
Title: Tcl/Tk Interface
Author: R Core Team
Maintainer: R Core Team <R-core@r-project.org>
Description: Interface and language bindings to Tcl/Tk GUI elements.
License: Part of R 3.2.0
Imports: utils
Built: R 3.2.0; armv5tel-unknown-linux-gnueabi; 2015-06-15 08:27:07 UTC; unix
