Package: grDevices
Version: 4.4.3
Priority: base
Title: The R Graphics Devices and Support for Colours and Fonts
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: Graphics devices and support for base and grid graphics.
License: Part of R 4.4.3
Suggests: KernSmooth
NeedsCompilation: yes
Built: R 4.4.3; armv7l-unknown-linux-gnueabihf; 2025-02-28 16:27:33 UTC; unix
