Package: tcltk
Version: 3.1.2
Priority: base
Title: Tcl/Tk Interface
Author: R Core Team
Maintainer: R Core Team <R-core@r-project.org>
Description: Interface and language bindings to Tcl/Tk GUI elements
License: Part of R 3.1.2
Imports: utils
Built: R 3.1.2; i686-pc-cygwin; 2015-01-30 23:56:22 UTC; unix
