Package: tcltk
Version: 3.2.3
Priority: base
Title: Tcl/Tk Interface
Author: R Core Team
Maintainer: R Core Team <R-core@r-project.org>
Description: Interface and language bindings to Tcl/Tk GUI elements.
License: Part of R 3.2.3
Imports: utils
Built: R 3.2.3; armv7l-unknown-linux-gnueabihf; 2015-12-12 12:32:12 UTC; unix
