Package: cluster
Version: 2.0.7-1
Date: 2018-04-06
Priority: recommended
Title: "Finding Groups in Data": Cluster Analysis Extended Rousseeuw et
        al.
Description: Methods for Cluster analysis.  Much extended the original from
	Peter Rousseeuw, Anja Struyf and Mia Hubert,
	based on Kaufman and Rousseeuw (1990) "Finding Groups in Data".
Maintainer: Martin Maechler <maechler@stat.math.ethz.ch>
Authors@R: c(person("Martin","Maechler", role=c("aut","cre"), email="maechler@stat.math.ethz.ch")
 ,person("Peter", "Rousseeuw", role="aut", email="peter.rousseeuw@kuleuven.be",
         comment = "Fortran original")
 ,person("Anja", "Struyf", role="aut", comment= "S original")
 ,person("Mia", "Hubert", role="aut", email= "Mia.Hubert@uia.ua.ac.be", comment= "S original")
 ,person("Kurt", "Hornik", role=c("trl", "ctb"),
 	email="Kurt.Hornik@R-project.org", comment="port to R; maintenance(1999-2000)")
 ,person("Matthias", "Studer", role="ctb")
 ,person("Pierre", "Roudier", role="ctb")
 ,person("Juan",   "Gonzalez", role="ctb")
 ,person("Kamil",  "Kozlowski", role="ctb")
 )
Depends: R (>= 3.2.0)
Imports: graphics, grDevices, stats, utils
Suggests: MASS, Matrix
SuggestsNote: two examples using MASS' cov.rob() and mvrnorm(); Matrix
        tools for testing
LazyLoad: yes
LazyData: yes
ByteCompile: yes
BuildResaveData: no
License: GPL (>= 2)
NeedsCompilation: yes
Packaged: 2018-04-06 21:30:04 UTC; maechler
Author: Martin Maechler [aut, cre],
  Peter Rousseeuw [aut] (Fortran original),
  Anja Struyf [aut] (S original),
  Mia Hubert [aut] (S original),
  Kurt Hornik [trl, ctb] (port to R; maintenance(1999-2000)),
  Matthias Studer [ctb],
  Pierre Roudier [ctb],
  Juan Gonzalez [ctb],
  Kamil Kozlowski [ctb]
Repository: CRAN
Date/Publication: 2018-04-13 08:06:02 UTC
Built: R 3.5.2; armv7l-unknown-linux-gnueabihf; 2019-01-13 14:46:52 UTC; unix
