Package: tcltk
Version: 3.4.3
Priority: base
Title: Tcl/Tk Interface
Author: R Core Team
Maintainer: R Core Team <R-core@r-project.org>
Description: Interface and language bindings to Tcl/Tk GUI elements.
License: Part of R 3.4.3
Imports: utils
NeedsCompilation: yes
Built: R 3.4.3; armv5tel-unknown-linux-gnueabi; 2017-11-30 19:54:23 UTC; unix
