Package: base
Version: 3.1.2
Priority: base
Title: The R Base Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: Base R functions
License: Part of R 3.1.2
Built: R 3.1.2; ; 2015-01-30 23:56:37 UTC; unix
