Package: cluster
Version: 2.1.0
Date: 2019-06-07
Priority: recommended
Title: "Finding Groups in Data": Cluster Analysis Extended Rousseeuw et
        al.
Description: Methods for Cluster analysis.  Much extended the original from
	Peter Rousseeuw, Anja Struyf and Mia Hubert,
	based on Kaufman and Rousseeuw (1990) "Finding Groups in Data".
Maintainer: Martin Maechler <maechler@stat.math.ethz.ch>
Authors@R: c(person("Martin","Maechler", role = c("aut","cre"),
	   email="maechler@stat.math.ethz.ch", comment = c(ORCID = "0000-0002-8685-9910"))
 ,person("Peter", "Rousseeuw", role="aut", email="peter.rousseeuw@kuleuven.be",
         comment = c("Fortran original", ORCID = "0000-0002-3807-5353"))
 ,person("Anja", "Struyf", role="aut", comment= "S original")
 ,person("Mia", "Hubert", role="aut", email= "Mia.Hubert@uia.ua.ac.be",
         comment = c("S original", ORCID = "0000-0001-6398-4850"))
 ,person("Kurt", "Hornik", role=c("trl", "ctb"),
         email="Kurt.Hornik@R-project.org",
         comment=c("port to R; maintenance(1999-2000)", ORCID="0000-0003-4198-9911"))
 ,person("Matthias", "Studer", role="ctb")
 ,person("Pierre", "Roudier", role="ctb")
 ,person("Juan",   "Gonzalez", role="ctb")
 ,person("Kamil",  "Kozlowski", role="ctb")
 ,person("Erich",  "Schubert", role="ctb", comment = c("fastpam options for pam()",
                                                       ORCID = "0000-0001-9143-4880"))
 ,person("Keefe",  "Murphy", role="ctb", comment = "volume.ellipsoid({d >= 3})")
 )
Depends: R (>= 3.3.0)
Imports: graphics, grDevices, stats, utils
Suggests: MASS, Matrix
SuggestsNote: MASS: two examples using cov.rob() and mvrnorm(); Matrix
        tools for testing
LazyLoad: yes
LazyData: yes
ByteCompile: yes
BuildResaveData: no
License: GPL (>= 2)
URL: https://svn.r-project.org/R-packages/trunk/cluster
NeedsCompilation: yes
Packaged: 2019-06-19 08:21:30 UTC; maechler
Author: Martin Maechler [aut, cre] (<https://orcid.org/0000-0002-8685-9910>),
  Peter Rousseeuw [aut] (Fortran original,
    <https://orcid.org/0000-0002-3807-5353>),
  Anja Struyf [aut] (S original),
  Mia Hubert [aut] (S original, <https://orcid.org/0000-0001-6398-4850>),
  Kurt Hornik [trl, ctb] (port to R; maintenance(1999-2000),
    <https://orcid.org/0000-0003-4198-9911>),
  Matthias Studer [ctb],
  Pierre Roudier [ctb],
  Juan Gonzalez [ctb],
  Kamil Kozlowski [ctb],
  Erich Schubert [ctb] (fastpam options for pam(),
    <https://orcid.org/0000-0001-9143-4880>),
  Keefe Murphy [ctb] (volume.ellipsoid({d >= 3}))
Repository: CRAN
Date/Publication: 2019-06-19 11:10:03 UTC
Built: R 3.6.2; armv7l-unknown-linux-gnueabihf; 2019-12-12 14:21:28 UTC; unix
