Package: tools
Version: 4.4.3
Priority: base
Title: Tools for Package Development
Author: R Core Team
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: Tools for package development, administration and documentation.
License: Part of R 4.4.3
Suggests: codetools, methods, xml2, curl, commonmark, knitr, xfun,
        mathjaxr, V8
NeedsCompilation: yes
Built: R 4.4.3; aarch64-unknown-linux-gnu; 2025-02-28 11:54:08 UTC; unix
