Package: codetools
Version: 0.2-9
Priority: recommended
Author: Luke Tierney <luke-tierney@uiowa.edu>
Description: Code analysis tools for R
Title: Code Analysis Tools for R
Depends: R (>= 2.1)
Maintainer: Luke Tierney <luke-tierney@uiowa.edu>
License: GPL
Packaged: 2014-08-18 19:30:24 UTC; luke
NeedsCompilation: no
Repository: CRAN
Date/Publication: 2014-08-21 10:48:50
Built: R 3.1.2; ; 2015-01-30 22:47:01 UTC; unix
