Package: tcltk
Version: 3.6.1
Priority: base
Title: Tcl/Tk Interface
Author: R Core Team
Maintainer: R Core Team <R-core@r-project.org>
Description: Interface and language bindings to Tcl/Tk GUI elements.
License: Part of R 3.6.1
Imports: utils
NeedsCompilation: yes
Built: R 3.6.1; armv7l-unknown-linux-gnueabihf; 2019-07-05 13:27:09 UTC; unix
