Package: codetools
Version: 0.2-10
Priority: recommended
Author: Luke Tierney <luke-tierney@uiowa.edu>
Description: Code analysis tools for R
Title: Code Analysis Tools for R
Depends: R (>= 2.1)
Maintainer: Luke Tierney <luke-tierney@uiowa.edu>
License: GPL
Packaged: 2015-01-16 21:15:47 UTC; luke
NeedsCompilation: no
Repository: CRAN
Date/Publication: 2015-01-17 21:57:23
Built: R 3.1.3; ; 2015-03-28 07:44:41 UTC; unix
