Package: graphics
Version: 4.6.0
Priority: base
Title: The R Graphics Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: R functions for base graphics.
Imports: grDevices
License: Part of R 4.6.0
NeedsCompilation: yes
Enhances: vcd
Encoding: UTF-8
Built: R 4.6.0; armv6--netbsdelf-eabihf; 2026-08-21 19:05:12 UTC; unix
