Package: tcltk
Version: 4.0.2
Priority: base
Title: Tcl/Tk Interface
Author: R Core Team
Maintainer: R Core Team <R-core@r-project.org>
Description: Interface and language bindings to Tcl/Tk GUI elements.
License: Part of R 4.0.2
Imports: utils
NeedsCompilation: yes
Built: R 4.0.2; aarch64-unknown-linux-gnu; 2020-06-22 09:23:46 UTC; unix
