Package: stats
Version: 3.1.2
Priority: base
Title: The R Stats Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: R statistical functions
License: Part of R 3.1.2
Built: R 3.1.2; i686-pc-cygwin; 2015-01-30 23:53:44 UTC; unix
