Package: methods
Version: 3.5.2
Priority: base
Imports: utils, stats
Title: Formal Methods and Classes
Author: R Core Team
Maintainer: R Core Team <R-core@r-project.org>
Description: Formally defined methods and classes for R objects,
  plus other programming tools, as described in the reference.
References: John M. Chambers (2008) ``Software for Data Analysis:
        Programming with R''; Springer NY.
License: Part of R 3.5.2
Suggests: codetools
NeedsCompilation: yes
Built: R 3.5.2; armv6l-unknown-linux-gnueabihf; 2019-01-13 17:10:32 UTC; unix
