Package: nlme
Version: 3.1-168
Date: 2025-03-31
Priority: recommended
Title: Linear and Nonlinear Mixed Effects Models
Authors@R: c(person("José", "Pinheiro", role = "aut", comment = "S version"),
             person("Douglas", "Bates", role = "aut", comment = "up to 2007"),
             person("Saikat", "DebRoy", role = "ctb", comment = "up to 2002"),
             person("Deepayan", "Sarkar", role = "ctb", comment = "up to 2005"),
             person("EISPACK authors", role = "ctb", comment = "src/rs.f"),
	     person("Siem", "Heisterkamp", role = "ctb", comment = "Author fixed sigma"),
             person("Bert", "Van Willigen",role = "ctb", comment = "Programmer fixed sigma"),
             person("Johannes", "Ranke", role = "ctb", comment = "varConstProp()"),
	     person("R Core Team", email = "R-core@R-project.org",
                    role = c("aut", "cre"), comment = c(ROR = "02zz1nj61")))
Contact: see 'MailingList'
Description: Fit and compare Gaussian linear and nonlinear mixed-effects models.
Depends: R (>= 3.6.0)
Imports: graphics, stats, utils, lattice
Suggests: MASS, SASmixed
LazyData: yes
Encoding: UTF-8
License: GPL (>= 2)
BugReports: https://bugs.r-project.org
MailingList: R-help@r-project.org
URL: https://svn.r-project.org/R-packages/trunk/nlme/
NeedsCompilation: yes
Packaged: 2025-03-31 11:19:09 UTC; ripley
Author: José Pinheiro [aut] (S version),
  Douglas Bates [aut] (up to 2007),
  Saikat DebRoy [ctb] (up to 2002),
  Deepayan Sarkar [ctb] (up to 2005),
  EISPACK authors [ctb] (src/rs.f),
  Siem Heisterkamp [ctb] (Author fixed sigma),
  Bert Van Willigen [ctb] (Programmer fixed sigma),
  Johannes Ranke [ctb] (varConstProp()),
  R Core Team [aut, cre] (02zz1nj61)
Maintainer: R Core Team <R-core@R-project.org>
Repository: CRAN
Date/Publication: 2025-03-31 11:21:01 UTC
Built: R 4.5.2; armv7--netbsdelf-eabihf; 2026-05-13 17:25:36 UTC; unix
