Package: nnet
Priority: recommended
Version: 7.3-19
Date: 2023-05-02
Depends: R (>= 3.0.0), stats, utils
Suggests: MASS
Authors@R: c(person("Brian", "Ripley", role = c("aut", "cre", "cph"),
                    email = "ripley@stats.ox.ac.uk"),
             person("William", "Venables", role = "cph"))
Description: Software for feed-forward neural networks with a single
  hidden layer, and for multinomial log-linear models.
Title: Feed-Forward Neural Networks and Multinomial Log-Linear Models
ByteCompile: yes
License: GPL-2 | GPL-3
URL: http://www.stats.ox.ac.uk/pub/MASS4/
NeedsCompilation: yes
Packaged: 2023-05-02 16:07:11 UTC; ripley
Author: Brian Ripley [aut, cre, cph],
  William Venables [cph]
Maintainer: Brian Ripley <ripley@stats.ox.ac.uk>
Repository: CRAN
Date/Publication: 2023-05-03 11:01:30 UTC
Built: R 4.4.2; i486--netbsdelf; 2025-12-23 09:37:09 UTC; unix
