Package: grDevices
Version: 3.1.2
Priority: base
Title: The R Graphics Devices and Support for Colours and Fonts
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: Graphics devices and support for base and grid graphics
License: Part of R 3.1.2
Built: R 3.1.2; x86_64-unknown-cygwin; 2015-01-30 23:11:16 UTC; unix
