Package: grDevices
Version: 3.1.3
Priority: base
Title: The R Graphics Devices and Support for Colours and Fonts
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: Graphics devices and support for base and grid graphics.
License: Part of R 3.1.3
Built: R 3.1.3; x86_64-unknown-cygwin; 2015-03-28 07:34:49 UTC; unix
