Package: base
Version: 3.4.2
Priority: base
Title: The R Base Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: Base R functions.
License: Part of R 3.4.2
Suggests: methods
Built: R 3.4.2; ; 2017-09-28 15:43:04 UTC; unix
