Title: Survival Analysis
Priority: recommended
Package: survival
Version: 3.8-6
Date: 2026-01-09
Depends: R (>= 3.5.0)
Imports: graphics, Matrix, methods, splines, stats, utils
LazyData: Yes
LazyDataCompression: xz
ByteCompile: Yes
Authors@R: c(person(c("Terry", "M"), "Therneau",
                    email="therneau.terry@mayo.edu",
	            role=c("aut", "cre")),
               person("Thomas", "Lumley", role=c("ctb", "trl"),
	       comment="original S->R port and R maintainer until 2009"),
	       person("Atkinson", "Elizabeth", role="ctb"),
	       person("Crowson", "Cynthia", role="ctb"))
Description: Contains the core survival analysis routines, including
	     definition of Surv objects, 
	     Kaplan-Meier and Aalen-Johansen (multi-state) curves, Cox models,
	     and parametric accelerated failure time models.
License: LGPL (>= 2)
URL: https://github.com/therneau/survival
NeedsCompilation: yes
Packaged: 2026-01-13 02:06:39 UTC; therneau
Author: Terry M Therneau [aut, cre],
  Thomas Lumley [ctb, trl] (original S->R port and R maintainer until
    2009),
  Atkinson Elizabeth [ctb],
  Crowson Cynthia [ctb]
Maintainer: Terry M Therneau <therneau.terry@mayo.edu>
Repository: CRAN
Date/Publication: 2026-01-16 09:00:02 UTC
Built: R 4.5.3; x86_64-unknown-openbsd7.9; 2026-06-21 11:13:31 UTC; unix
