Package: cluster
Version: 2.1.8.1
VersionNote: Last CRAN: 2.1.8 on 2024-12-10; 2.1.7 on 2024-12-06; 2.1.6
        on 2023-11-30; 2.1.5 on 2023-11-27
Date: 2025-03-11
Priority: recommended
Title: "Finding Groups in Data": Cluster Analysis Extended Rousseeuw et
        al.
Description: Methods for Cluster analysis.  Much extended the original from
	Peter Rousseeuw, Anja Struyf and Mia Hubert,
	based on Kaufman and Rousseeuw (1990) "Finding Groups in Data".
Maintainer: Martin Maechler <maechler@stat.math.ethz.ch>
Authors@R: c(person("Martin","Maechler", role = c("aut","cre"),
	   email="maechler@stat.math.ethz.ch", comment = c(ORCID = "0000-0002-8685-9910"))
 ,person("Peter", "Rousseeuw", role="aut", email="peter.rousseeuw@kuleuven.be",
         comment = c("Fortran original", ORCID = "0000-0002-3807-5353"))
 ,person("Anja", "Struyf", role="aut", comment= "S original")
 ,person("Mia", "Hubert", role="aut", email= "Mia.Hubert@uia.ua.ac.be",
         comment = c("S original", ORCID = "0000-0001-6398-4850"))
 ,person("Kurt", "Hornik", role=c("trl", "ctb"),
         email="Kurt.Hornik@R-project.org",
         comment=c("port to R; maintenance(1999-2000)", ORCID="0000-0003-4198-9911"))
 ,person("Matthias", "Studer", role="ctb")
 ,person("Pierre", "Roudier", role="ctb")
 ,person("Juan",   "Gonzalez", role="ctb")
 ,person("Kamil",  "Kozlowski", role="ctb")
 ,person("Erich",  "Schubert", role="ctb", comment = c("fastpam options for pam()",
                                                       ORCID = "0000-0001-9143-4880"))
 ,person("Keefe",  "Murphy", role="ctb", comment = "volume.ellipsoid({d >= 3})")
 #not yet ,person("Fischer-Rasmussen", "Kasper", role = "ctb", comment = "Gower distance for CLARA")
 )
Depends: R (>= 3.5.0)
Imports: graphics, grDevices, stats, utils
Suggests: MASS, Matrix
SuggestsNote: MASS: two examples using cov.rob() and mvrnorm(); Matrix
        tools for testing
Enhances: mvoutlier, fpc, ellipse, sfsmisc
EnhancesNote: xref-ed in man/*.Rd
LazyLoad: yes
LazyData: yes
ByteCompile: yes
BuildResaveData: no
License: GPL (>= 2)
URL: https://svn.r-project.org/R-packages/trunk/cluster/
NeedsCompilation: yes
Packaged: 2025-03-11 22:34:24 UTC; maechler
Author: Martin Maechler [aut, cre] (<https://orcid.org/0000-0002-8685-9910>),
  Peter Rousseeuw [aut] (Fortran original,
    <https://orcid.org/0000-0002-3807-5353>),
  Anja Struyf [aut] (S original),
  Mia Hubert [aut] (S original, <https://orcid.org/0000-0001-6398-4850>),
  Kurt Hornik [trl, ctb] (port to R; maintenance(1999-2000),
    <https://orcid.org/0000-0003-4198-9911>),
  Matthias Studer [ctb],
  Pierre Roudier [ctb],
  Juan Gonzalez [ctb],
  Kamil Kozlowski [ctb],
  Erich Schubert [ctb] (fastpam options for pam(),
    <https://orcid.org/0000-0001-9143-4880>),
  Keefe Murphy [ctb] (volume.ellipsoid({d >= 3}))
Repository: CRAN
Date/Publication: 2025-03-12 17:20:02 UTC
Built: R 4.5.2; x86_64--netbsd; 2026-06-02 12:58:56 UTC; unix
