Package: base
Version: 3.5.1
Priority: base
Title: The R Base Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: Base R functions.
License: Part of R 3.5.1
Suggests: methods
Built: R 3.5.1; ; 2018-08-09 01:47:19 UTC; unix
