Package: base
Version: 3.5.2
Priority: base
Title: The R Base Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: Base R functions.
License: Part of R 3.5.2
Suggests: methods
Built: R 3.5.2; ; 2019-01-13 14:37:54 UTC; unix
