Package: compiler
Version: 3.6.1
Priority: base
Title: The R Compiler Package
Author: Luke Tierney <luke-tierney@uiowa.edu>
Maintainer: R Core Team <R-core@r-project.org>
Description: Byte code compiler for R.
License: Part of R 3.6.1
Built: R 3.6.1; ; 2019-07-05 13:19:41 UTC; unix
