Package: graphics
Version: 3.4.3
Priority: base
Title: The R Graphics Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: R functions for base graphics.
Imports: grDevices
License: Part of R 3.4.3
NeedsCompilation: yes
Built: R 3.4.3; aarch64-unknown-linux-gnu; 2017-11-30 19:57:02 UTC; unix
