Package: tcltk
Version: 4.4.0
Priority: base
Title: Tcl/Tk Interface
Author: R Core Team
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: Interface and language bindings to Tcl/Tk GUI elements.
License: Part of R 4.4.0
Imports: utils
NeedsCompilation: yes
Built: R 4.4.0; i386-portbld-midnightbsd4.0; 2026-05-11 16:54:35 UTC; unix
