Package: graphics
Version: 3.5.3
Priority: base
Title: The R Graphics Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: R functions for base graphics.
Imports: grDevices
License: Part of R 3.5.3
NeedsCompilation: yes
Built: R 3.5.3; armv6l-unknown-linux-gnueabihf; 2019-04-07 21:34:44 UTC; unix
