Package: grDevices
Version: 3.5.3
Priority: base
Title: The R Graphics Devices and Support for Colours and Fonts
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: Graphics devices and support for base and grid graphics.
License: Part of R 3.5.3
Suggests: KernSmooth
NeedsCompilation: yes
Built: R 3.5.3; sparc64-unknown-openbsd6.5; 2019-04-25 03:21:43 UTC; unix
