Package: base
Version: 4.0.4
Priority: base
Title: The R Base Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: Base R functions.
License: Part of R 4.0.4
Suggests: methods
Built: R 4.0.4; ; 2021-02-15 10:04:58 UTC; unix
