Package: base
Version: 4.0.3
Priority: base
Title: The R Base Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: Base R functions.
License: Part of R 4.0.3
Suggests: methods
Built: R 4.0.3; ; 2020-10-10 16:11:54 UTC; unix
