Package: lattice
Version: 0.22-7
Date: 2025-03-31
Priority: recommended
Title: Trellis Graphics for R
Authors@R: c(person("Deepayan", "Sarkar", role = c("aut", "cre"),
	            email = "deepayan.sarkar@r-project.org",
		    comment = c(ORCID = "0000-0003-4107-1553")),
              person("Felix", "Andrews", role = "ctb"),
	      person("Kevin", "Wright", role = "ctb", comment = "documentation"),
	      person("Neil", "Klepeis", role = "ctb"),
	      person("Johan", "Larsson", role = "ctb", comment = "miscellaneous improvements"),
              person("Zhijian (Jason)", "Wen", role = "cph", comment = "filled contour code"),
              person("Paul", "Murrell", role = "ctb", email = "paul@stat.auckland.ac.nz"),
	      person("Stefan", "Eng", role = "ctb", comment = "violin plot improvements"),
	      person("Achim", "Zeileis", role = "ctb", comment = "modern colors"),
              person("Alexandre", "Courtiol", role = "ctb", comment = "generics for larrows, lpolygon, lrect and lsegments")
	      )
Description: A powerful and elegant high-level data visualization
  system inspired by Trellis graphics, with an emphasis on
  multivariate data. Lattice is sufficient for typical graphics needs,
  and is also flexible enough to handle most nonstandard requirements.
  See ?Lattice for an introduction.
Depends: R (>= 4.0.0)
Suggests: KernSmooth, MASS, latticeExtra, colorspace
Imports: grid, grDevices, graphics, stats, utils
Enhances: chron, zoo
LazyLoad: yes
LazyData: yes
License: GPL (>= 2)
URL: https://lattice.r-forge.r-project.org/
BugReports: https://github.com/deepayan/lattice/issues
NeedsCompilation: yes
Packaged: 2025-04-02 04:55:57 UTC; deepayan
Author: Deepayan Sarkar [aut, cre] (<https://orcid.org/0000-0003-4107-1553>),
  Felix Andrews [ctb],
  Kevin Wright [ctb] (documentation),
  Neil Klepeis [ctb],
  Johan Larsson [ctb] (miscellaneous improvements),
  Zhijian (Jason) Wen [cph] (filled contour code),
  Paul Murrell [ctb],
  Stefan Eng [ctb] (violin plot improvements),
  Achim Zeileis [ctb] (modern colors),
  Alexandre Courtiol [ctb] (generics for larrows, lpolygon, lrect and
    lsegments)
Maintainer: Deepayan Sarkar <deepayan.sarkar@r-project.org>
Repository: CRAN
Date/Publication: 2025-04-02 15:40:02 UTC
Built: R 4.5.2; x86_64--netbsd; 2026-06-02 12:58:46 UTC; unix
