Package: compiler
Version: 4.0.3
Priority: base
Title: The R Compiler Package
Author: Luke Tierney <luke-tierney@uiowa.edu>
Maintainer: R Core Team <R-core@r-project.org>
Description: Byte code compiler for R.
License: Part of R 4.0.3
Built: R 4.0.3; ; 2020-12-23 05:17:46 UTC; unix
