Package: graphics
Version: 3.6.1
Priority: base
Title: The R Graphics Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: R functions for base graphics.
Imports: grDevices
License: Part of R 3.6.1
NeedsCompilation: yes
Built: R 3.6.1; armv7l-unknown-linux-gnueabihf; 2019-07-05 13:24:06 UTC; unix
