Package: grid
Version: 4.6.0
Priority: base
Title: The Grid Graphics Package
Author: Paul Murrell <paul@stat.auckland.ac.nz>
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: A rewrite of the graphics layout capabilities, plus some
  support for interaction.
Imports: grDevices, utils
License: Part of R 4.6.0
NeedsCompilation: yes
Encoding: UTF-8
Built: R 4.6.0; armv6--netbsdelf-eabihf; 2026-08-21 19:09:25 UTC; unix
