Package: splines
Version: 4.6.0
Priority: base
Imports: graphics, stats
Title: Regression Spline Functions and Classes
Author: Douglas M. Bates <bates@stat.wisc.edu> and
 William N. Venables <Bill.Venables@csiro.au>
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: Regression spline functions and classes.
License: Part of R 4.6.0
Suggests: Matrix, methods
NeedsCompilation: yes
Encoding: UTF-8
Built: R 4.6.0; armv6--netbsdelf-eabihf; 2026-08-21 19:09:58 UTC; unix
