Package: graphics
Version: 4.0.2
Priority: base
Title: The R Graphics Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: R functions for base graphics.
Imports: grDevices
License: Part of R 4.0.2
NeedsCompilation: yes
Built: R 4.0.2; armv6l-unknown-linux-gnueabihf; 2020-06-22 09:23:57 UTC; unix
