Package: Matrix
Version: 1.2-11
Date: 2017-08-08
Priority: recommended
Title: Sparse and Dense Matrix Classes and Methods
Author: Douglas Bates <bates@stat.wisc.edu> and Martin Maechler
Maintainer: Martin Maechler <mmaechler+Matrix@gmail.com>
Contact: Doug and Martin <Matrix-authors@R-project.org>
Description: Classes and methods for dense and sparse matrices and
    operations on them using 'LAPACK' and 'SuiteSparse'.
Depends: R (>= 3.0.1)
Imports: methods, graphics, grid, stats, utils, lattice
Suggests: expm, MASS
Enhances: MatrixModels, graph, SparseM, sfsmisc
Encoding: UTF-8
LazyData: no
LazyDataNote: not possible, since we use data/*.R *and* our classes
BuildResaveData: no
License: GPL (>= 2) | file LICENCE
URL: http://Matrix.R-forge.R-project.org/
NeedsCompilation: yes
Packaged: 2017-08-10 15:34:00 UTC; maechler
Repository: CRAN
Date/Publication: 2017-08-21 10:31:45 UTC
Built: R 3.4.2; armv7l-unknown-linux-gnueabihf; 2017-09-28 15:45:25 UTC; unix
