Package: graphics
Version: 4.0.3
Priority: base
Title: The R Graphics Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: R functions for base graphics.
Imports: grDevices
License: Part of R 4.0.3
NeedsCompilation: yes
Built: R 4.0.3; armv5tel-unknown-linux-gnueabi; 2020-12-23 05:21:05 UTC; unix
