Package: class
Priority: recommended
Version: 7.3-23
Date: 2025-01-01
Depends: R (>= 3.0.0), stats, utils
Imports: MASS
Authors@R: c(person("Brian", "Ripley", role = c("aut", "cre", "cph"),
                    email = "Brian.Ripley@R-project.org"),
             person("William", "Venables", role = "cph"))
Description: Various functions for classification, including k-nearest
  neighbour, Learning Vector Quantization and Self-Organizing Maps.
Title: Functions for Classification
ByteCompile: yes
License: GPL-2 | GPL-3
URL: http://www.stats.ox.ac.uk/pub/MASS4/
NeedsCompilation: yes
Packaged: 2025-01-01 07:07:13 UTC; ripley
Author: Brian Ripley [aut, cre, cph],
  William Venables [cph]
Maintainer: Brian Ripley <Brian.Ripley@R-project.org>
Repository: CRAN
Date/Publication: 2025-01-01 10:25:33 UTC
Built: R 4.5.2; aarch64--netbsd; 2026-06-16 12:42:18 UTC; unix
