Package: graphics
Version: 4.0.0
Priority: base
Title: The R Graphics Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: R functions for base graphics.
Imports: grDevices
License: Part of R 4.0.0
NeedsCompilation: yes
Built: R 4.0.0; aarch64-unknown-linux-gnu; 2020-04-24 12:48:16 UTC; unix
