Package: tcltk
Version: 4.6.0
Priority: base
Title: Tcl/Tk Interface
Author: R Core Team
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: Interface and language bindings to Tcl/Tk GUI elements.
License: Part of R 4.6.0
Imports: utils
NeedsCompilation: yes
Built: R 4.6.0; armv6--netbsdelf-eabihf; 2026-08-21 19:10:08 UTC; unix
