Package: parallel
Version: 3.1.2
Priority: base
Title: Support for Parallel computation in R
Author: R Core Team
Maintainer: R Core Team <R-core@r-project.org>
Description: Support for parallel computation, including by forking
   (taken from package multicore), by sockets (taken from package snow)
   and random-number generation.
License: Part of R 3.1.2
Imports: tools
Enhances: snow, nws, Rmpi
Built: R 3.1.2; i686-pc-cygwin; 2015-01-30 23:56:30 UTC; unix
