Package: class
Priority: recommended
Version: 7.3-19
Date: 2021-04-17
Depends: R (>= 3.0.0), stats, utils
Imports: MASS
Authors@R: c(person("Brian", "Ripley", role = c("aut", "cre", "cph"),
                    email = "ripley@stats.ox.ac.uk"),
             person("William", "Venables", role = "cph"))
Description: Various functions for classification, including k-nearest
  neighbour, Learning Vector Quantization and Self-Organizing Maps.
Title: Functions for Classification
ByteCompile: yes
License: GPL-2 | GPL-3
URL: http://www.stats.ox.ac.uk/pub/MASS4/
NeedsCompilation: yes
Packaged: 2021-05-03 08:31:22 UTC; ripley
Author: Brian Ripley [aut, cre, cph],
  William Venables [cph]
Maintainer: Brian Ripley <ripley@stats.ox.ac.uk>
Repository: CRAN
Date/Publication: 2021-05-03 09:03:30 UTC
Built: R 4.1.2; aarch64-unknown-linux-gnu; 2022-02-01 03:20:34 UTC; unix
