Package: mgcv
Version: 1.8-6
Author: Simon Wood <simon.wood@r-project.org>
Maintainer: Simon Wood <simon.wood@r-project.org>
Title: Mixed GAM Computation Vehicle with GCV/AIC/REML Smoothness
        Estimation
Description: GAMs, GAMMs and other generalized ridge regression with 
             multiple smoothing parameter estimation by GCV, REML or UBRE/AIC. 
             Includes a gam() function, a wide variety of smoothers, JAGS 
             support and distributions beyond the exponential family.
Priority: recommended
Depends: R (>= 2.14.0), nlme (>= 3.1-64)
Imports: methods, stats, graphics, Matrix
Suggests: splines, parallel, survival, MASS
LazyLoad: yes
ByteCompile: yes
License: GPL (>= 2)
Packaged: 2015-03-30 13:02:31 UTC; sw283
NeedsCompilation: yes
Repository: CRAN
Date/Publication: 2015-03-31 12:45:12
Built: R 3.2.0; armv5tel-unknown-linux-gnueabi; 2015-06-15 08:38:18 UTC; unix
