Package: MASS
Priority: recommended
Version: 7.3-65
Date: 2025-02-19
Revision: $Rev: 3681 $
Depends: R (>= 4.4.0), grDevices, graphics, stats, utils
Imports: methods
Suggests: lattice, nlme, nnet, survival
Authors@R: c(person("Brian", "Ripley", role = c("aut", "cre", "cph"),
                    email = "Brian.Ripley@R-project.org"),
	     person("Bill", "Venables", role = c("aut", "cph")),
	     person(c("Douglas", "M."), "Bates", role = "ctb"),
	     person("Kurt", "Hornik", role = "trl",
                     comment = "partial port ca 1998"),
	     person("Albrecht", "Gebhardt", role = "trl",
                     comment = "partial port ca 1998"),
	     person("David", "Firth", role = "ctb",
	            comment = "support functions for polr"))
Description: Functions and datasets to support Venables and Ripley,
  "Modern Applied Statistics with S" (4th edition, 2002).
Title: Support Functions and Datasets for Venables and Ripley's MASS
LazyData: yes
ByteCompile: yes
License: GPL-2 | GPL-3
URL: http://www.stats.ox.ac.uk/pub/MASS4/
Contact: <MASS@stats.ox.ac.uk>
NeedsCompilation: yes
Packaged: 2025-02-19 08:49:43 UTC; ripley
Author: Brian Ripley [aut, cre, cph],
  Bill Venables [aut, cph],
  Douglas M. Bates [ctb],
  Kurt Hornik [trl] (partial port ca 1998),
  Albrecht Gebhardt [trl] (partial port ca 1998),
  David Firth [ctb] (support functions for polr)
Maintainer: Brian Ripley <Brian.Ripley@R-project.org>
Repository: CRAN
Date/Publication: 2025-02-28 17:44:52 UTC
Built: R 4.5.2; armv7--netbsdelf-eabihf; 2026-04-12 20:19:40 UTC; unix
