Package: datasets
Version: 3.2.3
Priority: base
Title: The R Datasets Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: Base R datasets.
License: Part of R 3.2.3
Built: R 3.2.3; ; 2015-12-12 12:30:53 UTC; unix
