Package: datasets
Version: 3.2.4
Priority: base
Title: The R Datasets Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: Base R datasets.
License: Part of R 3.2.4
Built: R 3.2.4; ; 2016-04-05 20:59:36 UTC; unix
