Package: tcltk
Version: 4.6.1
Priority: base
Title: Tcl/Tk Interface
Author: R Core Team
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: Interface and language bindings to Tcl/Tk GUI elements.
License: Part of R 4.6.1
Imports: utils
NeedsCompilation: yes
Built: R 4.6.1; x86_64-pc-linux-gnu; 2026-07-01 19:48:57 UTC; unix
