Title: Survival Analysis
Maintainer: Terry M Therneau <therneau.terry@mayo.edu>
Priority: recommended
Package: survival
Version: 2.38-3
Depends: R (>= 2.13.0), graphics, stats
Imports: splines, methods
Suggests: cmprsk
LazyData: Yes
LazyLoad: Yes
ByteCompile: Yes
Authors@R: c(person(c("Terry", "M"), "Therneau",
                    email="therneau.terry@mayo.edu",
	            role=c("aut", "cre")),
             person("Thomas", "Lumley", role=c("ctb", "trl"),
	       comment="original S->R port and maintainer until 2009"))
Description: Contains the core survival analysis routines, including
	     definition of Surv objects, 
	     Kaplan-Meier and Aalen-Johansen (multi-state) curves, Cox models,
	     and parametric accelerated failure time models.
License: LGPL (>= 2)
NeedsCompilation: yes
Packaged: 2015-07-01 21:33:35 UTC; therneau
Author: Terry M Therneau [aut, cre],
  Thomas Lumley [ctb, trl] (original S->R port and maintainer until 2009)
Repository: CRAN
Date/Publication: 2015-07-02 08:40:39
Built: R 3.2.3; armv7l-unknown-linux-gnueabihf; 2015-12-12 12:39:40 UTC; unix
