Package: stats
Version: 3.2.0
Priority: base
Title: The R Stats Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: R statistical functions.
License: Part of R 3.2.0
Built: R 3.2.0; armv5tel-unknown-linux-gnueabi; 2015-06-15 08:23:59 UTC; unix
