Package: utils
Version: 3.4.2
Priority: base
Title: The R Utils Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: R utility functions.
License: Part of R 3.4.2
Suggests: methods, XML
NeedsCompilation: yes
Built: R 3.4.2; aarch64-unknown-linux-gnu; 2017-09-28 15:40:49 UTC; unix
