Package: utils
Version: 3.5.2
Priority: base
Title: The R Utils Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: R utility functions.
License: Part of R 3.5.2
Suggests: methods, xml2, commonmark
NeedsCompilation: yes
Built: R 3.5.2; armv6l-unknown-linux-gnueabihf; 2019-01-13 17:07:45 UTC; unix
