Package: datasets
Version: 3.6.1
Priority: base
Title: The R Datasets Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: Base R datasets.
License: Part of R 3.6.1
Built: R 3.6.1; ; 2019-11-14 16:08:30 UTC; unix
