Package: stats
Version: 3.2.4
Priority: base
Title: The R Stats Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: R statistical functions.
License: Part of R 3.2.4
Suggests: MASS, Matrix, SuppDists, methods, stats4
Built: R 3.2.4; armv7l-unknown-linux-gnueabihf; 2016-04-05 20:57:58 UTC; unix
