Package: graphics
Version: 4.3.2
Priority: base
Title: The R Graphics Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: R functions for base graphics.
Imports: grDevices
License: Part of R 4.3.2
NeedsCompilation: yes
Built: R 4.3.2; aarch64-unknown-linux-gnu; 2023-10-31 11:09:50 UTC; unix
