Package: datasets
Version: 4.0.3
Priority: base
Title: The R Datasets Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: Base R datasets.
License: Part of R 4.0.3
Built: R 4.0.3; ; 2020-12-23 05:22:58 UTC; unix
