Package: datasets
Version: 4.0.4
Priority: base
Title: The R Datasets Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: Base R datasets.
License: Part of R 4.0.4
Built: R 4.0.4; ; 2021-02-15 10:03:14 UTC; unix
