Package: utils
Version: 3.6.3
Priority: base
Title: The R Utils Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: R utility functions.
License: Part of R 3.6.3
Suggests: methods, xml2, commonmark
NeedsCompilation: yes
Built: R 3.6.3; armv7l-unknown-linux-gnueabihf; 2020-02-29 09:51:54 UTC; unix
