Package: boot
Priority: recommended
Version: 1.3-13
Date: 2014-09-29
Authors@R: c(person("Angelo", "Canty", role = "aut",
	             email = "cantya@mcmaster.ca"),
             person("Brian", "Ripley", role = c("aut", "trl", "cre"),
                    email = "ripley@stats.ox.ac.uk",
		    comment = "author of parallel support"))
Maintainer: Brian Ripley <ripley@stats.ox.ac.uk>
Note: Maintainers are not available to give advice on using a package
        they did not author.
Description: functions and datasets for bootstrapping from the
  book "Bootstrap Methods and Their Applications" by A. C. Davison and 
  D. V. Hinkley (1997, CUP).
Title: Bootstrap Functions (originally by Angelo Canty for S)
Depends: R (>= 3.0.0), graphics, stats
Suggests: MASS, survival
LazyData: yes
ByteCompile: yes
License: Unlimited
Packaged: 2014-09-29 09:08:42 UTC; ripley
Author: Angelo Canty [aut],
  Brian Ripley [aut, trl, cre] (author of parallel support)
NeedsCompilation: no
Repository: CRAN
Date/Publication: 2014-09-30 09:00:27
Built: R 3.1.2; ; 2015-01-31 00:03:12 UTC; unix
