Package: Matrix
Version: 1.4-1
Date: 2022-03-21
Priority: recommended
Title: Sparse and Dense Matrix Classes and Methods
Contact: Matrix-authors@R-project.org
Maintainer: Martin Maechler <mmaechler+Matrix@gmail.com>
Authors@R: c(person("Douglas","Bates", role="aut")
 , person("Martin","Maechler", role = c("aut","cre"), email="mmaechler+Matrix@gmail.com",
          comment = c(ORCID = "0000-0002-8685-9910"))
 , person("Mikael", "Jagan", role="aut",
           comment = "new 'packedMatrix' class plus subset/diag/.. methods in C")
 , person("Timothy A.", "Davis", role="ctb",
           comment = c("SuiteSparse and 'cs' C libraries, notably CHOLMOD, AMD;
		       collaborators listed in
			dir(pattern = '^[A-Z]+[.]txt$', full.names=TRUE,
			    system.file('doc', 'SuiteSparse', package='Matrix'))"))
 , person("Jens", "Oehlschlägel", role="ctb", comment="initial nearPD()")
 , person("Jason", "Riedy", role="ctb",
          comment = c("condest() and onenormest() for octave",
 	  	      "Copyright: Regents of the University of California"))
 , person("R Core Team", role = "ctb", comment="base R matrix implementation")
 )
Description: A rich hierarchy of matrix classes, including triangular,
   symmetric, and diagonal matrices, both dense and sparse and with
   pattern, logical and numeric entries.   Numerous methods for and
   operations on these matrices, using 'LAPACK' and 'SuiteSparse' libraries.
Depends: R (>= 3.5.0)
Imports: methods, graphics, grid, stats, utils, lattice
Suggests: expm, MASS
Enhances: MatrixModels, graph, SparseM, sfsmisc, igraph, maptools, sp,
        spdep
EnhancesNote: line 2: for "Rd xrefs"
Encoding: UTF-8
LazyData: no
LazyDataNote: not possible, since we use data/*.R *and* our classes
BuildResaveData: no
License: GPL (>= 2) | file LICENCE
URL: https://Matrix.R-forge.R-project.org/,
        https://Matrix.R-forge.R-project.org/doxygen/
BugReports: https://R-forge.R-project.org/tracker/?atid=294?group_id=61
Author: Douglas Bates [aut],
  Martin Maechler [aut, cre] (<https://orcid.org/0000-0002-8685-9910>),
  Mikael Jagan [aut] (new 'packedMatrix' class plus subset/diag/..
    methods in C),
  Timothy A. Davis [ctb] (SuiteSparse and 'cs' C libraries, notably
    CHOLMOD, AMD; collaborators listed in dir(pattern =
    '^[A-Z]+[.]txt$', full.names=TRUE, system.file('doc',
    'SuiteSparse', package='Matrix'))),
  Jens Oehlschlägel [ctb] (initial nearPD()),
  Jason Riedy [ctb] (condest() and onenormest() for octave, Copyright:
    Regents of the University of California),
  R Core Team [ctb] (base R matrix implementation)
Repository: CRAN
Repository/R-Forge/Project: matrix
Repository/R-Forge/Revision: 3446
Repository/R-Forge/DateTimeStamp: 2022-03-21 17:52:00
Date/Publication: 2022-03-23 16:30:08 UTC
NeedsCompilation: yes
Packaged: 2022-03-21 18:13:03 UTC; rforge
Built: R 4.2.0; armv7l-unknown-linux-gnueabihf; 2022-04-22 13:31:51 UTC; unix
