Package: methods
Version: 4.6.0
Priority: base
Imports: utils, stats
Title: Formal Methods and Classes
Author: R Core Team
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: Formally defined methods and classes for R objects,
  plus other programming tools, as described in the reference.
References: John M. Chambers (2008) ``Software for Data Analysis:
        Programming with R''; Springer NY.
License: Part of R 4.6.0
Suggests: codetools
NeedsCompilation: yes
Encoding: UTF-8
Built: R 4.6.0; aarch64-unknown-linux-gnu; 2026-04-26 15:19:59 UTC; unix
