Package: splines
Version: 3.4.3
Priority: base
Imports: graphics, stats
Title: Regression Spline Functions and Classes
Author: Douglas M. Bates <bates@stat.wisc.edu> and
 William N. Venables <Bill.Venables@csiro.au>
Maintainer: R Core Team <R-core@r-project.org>
Description: Regression spline functions and classes.
License: Part of R 3.4.3
Suggests: Matrix, methods
NeedsCompilation: yes
Built: R 3.4.3; aarch64-unknown-linux-gnu; 2017-11-30 20:02:00 UTC; unix
