Package: stats4
Title: Statistical Functions using S4 Classes
Version: 4.5.2
Priority: base
Author: R Core Team and contributors worldwide
Description: Statistical Functions using S4 classes.
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Imports: graphics, methods, stats
License: Part of R 4.5.2
Built: R 4.5.2; ; 2026-06-16 12:40:28 UTC; unix
