Package: mgcv
Version: 1.8-3
Author: Simon Wood <simon.wood@r-project.org>
Maintainer: Simon Wood <simon.wood@r-project.org>
Title: Mixed GAM Computation Vehicle with GCV/AIC/REML smoothness
        estimation
Description: Routines for GAMs and other generalized ridge regression  
             with multiple smoothing parameter selection by GCV, REML  
             or UBRE/AIC. Also GAMMs. Includes a gam() function.
Priority: recommended
Depends: R (>= 2.14.0), nlme (>= 3.1-64)
Imports: methods, stats, graphics, Matrix
Suggests: splines, parallel, survival, MASS
LazyLoad: yes
ByteCompile: yes
License: GPL (>= 2)
Packaged: 2014-08-29 10:10:03 UTC; sw283
NeedsCompilation: yes
Repository: CRAN
Date/Publication: 2014-08-29 22:07:21
Built: R 3.1.2; i686-pc-cygwin; 2015-01-31 00:04:35 UTC; unix
