Package: grDevices
Version: 4.6.1
Priority: base
Title: The R Graphics Devices and Support for Colours and Fonts
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: Graphics devices and support for base and grid graphics.
License: Part of R 4.6.1
Suggests: KernSmooth
NeedsCompilation: yes
Encoding: UTF-8
Built: R 4.6.1; x86_64-unknown-openbsd7.9; 2026-07-16 12:10:19 UTC; unix
