Package: rpart
Priority: recommended
Version: 4.1.27
Date: 2026-03-26
Authors@R: c(person("Terry", "Therneau", role = "aut",
	            email = "therneau@mayo.edu"),
             person("Beth", "Atkinson", role = c("aut", "cre"),
	            email = "atkinson@mayo.edu"),
             person("Brian", "Ripley", role = "trl",
                    email = "Brian.Ripley@R-project.org",
		    comment = "producer of the initial R port, maintainer 1999-2017"))
Description: Recursive partitioning for classification, 
  regression and survival trees.  An implementation of most of the 
  functionality of the 1984 book by Breiman, Friedman, Olshen and Stone.
Title: Recursive Partitioning and Regression Trees
Depends: R (>= 2.15.0), graphics, stats, grDevices
Suggests: survival
License: GPL-2 | GPL-3
LazyData: yes
ByteCompile: yes
NeedsCompilation: yes
Author: Terry Therneau [aut],
  Beth Atkinson [aut, cre],
  Brian Ripley [trl] (producer of the initial R port, maintainer
    1999-2017)
Maintainer: Beth Atkinson <atkinson@mayo.edu>
Repository: CRAN
URL: https://github.com/bethatkinson/rpart,
        https://cran.r-project.org/package=rpart
BugReports: https://github.com/bethatkinson/rpart/issues
Packaged: 2026-03-26 15:46:48 UTC; atkinson
Date/Publication: 2026-03-27 06:10:02 UTC
Built: R 4.6.0; armv6--netbsdelf-eabihf; 2026-08-21 19:12:29 UTC; unix
