Package: nnet
Priority: recommended
Version: 7.3-17
Date: 2022-01-12
Depends: R (>= 3.0.0), stats, utils
Suggests: MASS
Authors@R: c(person("Brian", "Ripley", role = c("aut", "cre", "cph"),
                    email = "ripley@stats.ox.ac.uk"),
             person("William", "Venables", role = "cph"))
Description: Software for feed-forward neural networks with a single
  hidden layer, and for multinomial log-linear models.
Title: Feed-Forward Neural Networks and Multinomial Log-Linear Models
ByteCompile: yes
License: GPL-2 | GPL-3
URL: http://www.stats.ox.ac.uk/pub/MASS4/
NeedsCompilation: yes
Packaged: 2022-01-13 05:06:39 UTC; ripley
Author: Brian Ripley [aut, cre, cph],
  William Venables [cph]
Maintainer: Brian Ripley <ripley@stats.ox.ac.uk>
Repository: CRAN
Date/Publication: 2022-01-16 10:24:24 UTC
Built: R 4.1.3; armv7l-unknown-linux-gnueabihf; 2022-03-10 14:41:07 UTC; unix
