Package: splines
Version: 4.0.0
Priority: base
Imports: graphics, stats
Title: Regression Spline Functions and Classes
Author: Douglas M. Bates <bates@stat.wisc.edu> and
 William N. Venables <Bill.Venables@csiro.au>
Maintainer: R Core Team <R-core@r-project.org>
Description: Regression spline functions and classes.
License: Part of R 4.0.0
Suggests: Matrix, methods
NeedsCompilation: yes
Built: R 4.0.0; armv7l-unknown-linux-gnueabihf; 2020-05-01 14:27:46 UTC; unix
