Package: tools
Version: 3.1.2
Priority: base
Title: Tools for Package Development
Author: R Core Team
Maintainer: R Core Team <R-core@r-project.org>
Description: Tools for package development, administration and documentation
License: Part of R 3.1.2
Built: R 3.1.2; i686-pc-cygwin; 2015-01-30 23:51:39 UTC; unix
