Package: base
Version: 4.4.1
Priority: base
Title: The R Base Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: Base R functions.
License: Part of R 4.4.1
Suggests: methods
Enhances: chron, date, round
Built: R 4.4.1; ; 2024-06-14 14:54:52 UTC; unix
