Package: compiler
Version: 4.5.0
Priority: base
Title: The R Compiler Package
Author: Luke Tierney <luke-tierney@uiowa.edu>
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: Byte code compiler for R.
License: Part of R 4.5.0
Built: R 4.5.0; ; 2025-04-11 15:35:08 UTC; unix
