Package: utils
Version: 4.2.2
Priority: base
Title: The R Utils Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: R utility functions.
License: Part of R 4.2.2
Suggests: methods, xml2, commonmark, knitr
NeedsCompilation: yes
Built: R 4.2.2; armv7l-unknown-linux-gnueabihf; 2023-01-05 06:10:11 UTC; unix
