Package: datasets
Version: 4.3.2
Priority: base
Title: The R Datasets Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: Base R datasets.
License: Part of R 4.3.2
Built: R 4.3.2; ; 2023-10-31 11:11:07 UTC; unix
