Package: utils
Version: 3.1.3
Priority: base
Title: The R Utils Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: R utility functions.
License: Part of R 3.1.3
Built: R 3.1.3; x86_64-unknown-cygwin; 2015-03-28 07:34:28 UTC; unix
