Package: spatial
Priority: recommended
Version: 7.3-18
Date: 2025-01-01
Depends: R (>= 3.0.0), graphics, stats, utils
Suggests: MASS
Authors@R: c(person("Brian", "Ripley", role = c("aut", "cre", "cph"),
                    email = "Brian.Ripley@R-project.org"),
	     person("Roger", "Bivand", role = "ctb"),
             person("William", "Venables", role = "cph"))
Description: Functions for kriging and point pattern analysis.
Title: Functions for Kriging and Point Pattern Analysis
LazyLoad: yes
ByteCompile: yes
License: GPL-2 | GPL-3
URL: http://www.stats.ox.ac.uk/pub/MASS4/
NeedsCompilation: yes
Packaged: 2025-01-01 07:07:13 UTC; ripley
Author: Brian Ripley [aut, cre, cph],
  Roger Bivand [ctb],
  William Venables [cph]
Maintainer: Brian Ripley <Brian.Ripley@R-project.org>
Repository: CRAN
Date/Publication: 2025-01-01 10:25:46 UTC
Built: R 4.5.2; sparc64-unknown-openbsd7.9; 2026-04-27 19:28:34 UTC; unix
