Package: parallel
Version: 4.0.3
Priority: base
Title: Support for Parallel computation in R
Author: R Core Team
Maintainer: R Core Team <R-core@r-project.org>
Description: Support for parallel computation, including by forking
   (taken from package multicore), by sockets (taken from package snow)
   and random-number generation.
License: Part of R 4.0.3
Imports: tools, compiler
Suggests: methods
Enhances: snow, nws, Rmpi
NeedsCompilation: yes
Built: R 4.0.3; aarch64-unknown-linux-gnu; 2020-12-22 16:58:22 UTC; unix
