Package: parallel
Version: 4.6.0
Priority: base
Title: Support for Parallel Computation in R
Author: R Core Team
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: Support for parallel computation, including by forking
   (taken from package multicore), by sockets (taken from package snow)
   and random-number generation.
License: Part of R 4.6.0
Imports: tools, compiler
Suggests: methods
Enhances: snow, Rmpi, mirai, parallelly
NeedsCompilation: yes
Encoding: UTF-8
Built: R 4.6.0; armv6--netbsdelf-eabihf; 2026-08-21 19:10:15 UTC; unix
