Package: base
Version: 4.6.0
Priority: base
Title: The R Base Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: Base R functions.
License: Part of R 4.6.0
Suggests: methods
Enhances: chron, date, round
Encoding: UTF-8
Built: R 4.6.0; ; 2026-04-28 17:57:23 UTC; unix
