Package: foreign
Priority: recommended
Version: 0.8-90
Date: 2025-03-31
Title: Read Data Stored by 'Minitab', 'S', 'SAS', 'SPSS', 'Stata',
        'Systat', 'Weka', 'dBase', ...
Depends: R (>= 4.0.0)
Imports: methods, utils, stats
Authors@R: c(
    person("R Core Team", email = "R-core@R-project.org",
 	   role = c("aut", "cph", "cre"),
	   comment = c(ROR = "02zz1nj61")),
    person("Roger", "Bivand", role = c("ctb", "cph")),
    person(c("Vincent", "J."), "Carey", role = c("ctb", "cph")),
    person("Saikat", "DebRoy", role = c("ctb", "cph")),
    person("Stephen", "Eglen", role = c("ctb", "cph")),
    person("Rajarshi", "Guha", role = c("ctb", "cph")),
    person("Swetlana", "Herbrandt", role = "ctb"),
    person("Nicholas", "Lewin-Koh", role = c("ctb", "cph")),
    person("Mark", "Myatt", role = c("ctb", "cph")),
    person("Michael", "Nelson", role = "ctb"),
    person("Ben", "Pfaff", role = "ctb"),
    person("Brian", "Quistorff", role = "ctb"),
    person("Frank", "Warmerdam", role = c("ctb", "cph")),
    person("Stephen", "Weigand", role = c("ctb", "cph")),
    person("Free Software Foundation, Inc.", role = "cph"))
Contact: see 'MailingList'
Copyright: see file COPYRIGHTS
Description: Reading and writing data stored by some versions of
	'Epi Info', 'Minitab', 'S', 'SAS', 'SPSS', 'Stata', 'Systat', 'Weka',
	and for reading and writing some 'dBase' files.
ByteCompile: yes
Biarch: yes
License: GPL (>= 2)
BugReports: https://bugs.r-project.org
MailingList: R-help@r-project.org
URL: https://svn.r-project.org/R-packages/trunk/foreign/
NeedsCompilation: yes
Packaged: 2025-03-31 11:19:10 UTC; ripley
Author: R Core Team [aut, cph, cre] (02zz1nj61),
  Roger Bivand [ctb, cph],
  Vincent J. Carey [ctb, cph],
  Saikat DebRoy [ctb, cph],
  Stephen Eglen [ctb, cph],
  Rajarshi Guha [ctb, cph],
  Swetlana Herbrandt [ctb],
  Nicholas Lewin-Koh [ctb, cph],
  Mark Myatt [ctb, cph],
  Michael Nelson [ctb],
  Ben Pfaff [ctb],
  Brian Quistorff [ctb],
  Frank Warmerdam [ctb, cph],
  Stephen Weigand [ctb, cph],
  Free Software Foundation, Inc. [cph]
Maintainer: R Core Team <R-core@R-project.org>
Repository: CRAN
Date/Publication: 2025-03-31 11:21:11 UTC
Built: R 4.5.0; aarch64-unknown-linux-gnu; 2025-04-11 15:40:01 UTC; unix
