Package: tools
Version: 3.5.3
Priority: base
Title: Tools for Package Development
Author: R Core Team
Maintainer: R Core Team <R-core@r-project.org>
Description: Tools for package development, administration and documentation.
License: Part of R 3.5.3
Suggests: codetools, methods, xml2, curl
NeedsCompilation: yes
Built: R 3.5.3; sparc64-unknown-openbsd6.5; 2019-04-25 03:16:14 UTC; unix
