Package: graphics
Version: 4.6.1
Priority: base
Title: The R Graphics Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: R functions for base graphics.
Imports: grDevices
License: Part of R 4.6.1
NeedsCompilation: yes
Enhances: vcd
Encoding: UTF-8
Built: R 4.6.1; aarch64-unknown-linux-gnu; 2026-06-24 11:16:15 UTC; unix
