Package: splines
Version: 4.0.5
Priority: base
Imports: graphics, stats
Title: Regression Spline Functions and Classes
Author: Douglas M. Bates <bates@stat.wisc.edu> and
 William N. Venables <Bill.Venables@csiro.au>
Maintainer: R Core Team <R-core@r-project.org>
Description: Regression spline functions and classes.
License: Part of R 4.0.5
Suggests: Matrix, methods
NeedsCompilation: yes
Built: R 4.0.5; armv7l-unknown-linux-gnueabihf; 2021-04-22 14:19:05 UTC; unix
