Package: tools
Version: 4.0.2
Priority: base
Title: Tools for Package Development
Author: R Core Team
Maintainer: R Core Team <R-core@r-project.org>
Description: Tools for package development, administration and documentation.
License: Part of R 4.0.2
Suggests: codetools, methods, xml2, curl, commonmark
NeedsCompilation: yes
Built: R 4.0.2; x86_64-unknown-openbsd6.8; 2020-09-30 23:22:20 UTC; unix
