Package: utils
Version: 4.0.2
Priority: base
Title: The R Utils Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: R utility functions.
License: Part of R 4.0.2
Suggests: methods, xml2, commonmark
NeedsCompilation: yes
Built: R 4.0.2; x86_64-unknown-openbsd6.8; 2020-09-30 23:23:12 UTC; unix
