Package: datasets
Version: 4.0.2
Priority: base
Title: The R Datasets Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: Base R datasets.
License: Part of R 4.0.2
Built: R 4.0.2; ; 2020-09-30 23:24:34 UTC; unix
