Package: datasets
Version: 4.1.0
Priority: base
Title: The R Datasets Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: Base R datasets.
License: Part of R 4.1.0
Built: R 4.1.0; ; 2021-05-28 10:52:44 UTC; unix
