Package: mgcv
Version: 1.9-3
Authors@R: person(given = "Simon",
                    family = "Wood",
                    role = c("aut", "cre"),
                    email = "simon.wood@r-project.org")
Title: Mixed GAM Computation Vehicle with Automatic Smoothness
        Estimation
Description: Generalized additive (mixed) models, some of their extensions and 
             other generalized ridge regression with multiple smoothing 
             parameter estimation by (Restricted) Marginal Likelihood, 
             Generalized Cross Validation and similar, or using iterated 
             nested Laplace approximation for fully Bayesian inference. See 
             Wood (2017) <doi:10.1201/9781315370279> for an overview. 
             Includes a gam() function, a wide variety of smoothers, 'JAGS' 
             support and distributions beyond the exponential family. 
Priority: recommended
Depends: R (>= 3.6.0), nlme (>= 3.1-64)
Imports: methods, stats, graphics, Matrix, splines, utils
Suggests: parallel, survival, MASS
LazyLoad: yes
ByteCompile: yes
License: GPL (>= 2)
NeedsCompilation: yes
Packaged: 2025-04-03 13:22:13 UTC; sw283
Author: Simon Wood [aut, cre]
Maintainer: Simon Wood <simon.wood@r-project.org>
Repository: CRAN
Date/Publication: 2025-04-04 05:40:02 UTC
Built: R 4.5.2; armv7--netbsdelf-eabihf; 2026-06-17 04:09:29 UTC; unix
