Package: utils
Version: 4.4.0
Priority: base
Title: The R Utils Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: R utility functions.
License: Part of R 4.4.0
Suggests: methods, xml2, commonmark, knitr, jsonlite
NeedsCompilation: yes
Built: R 4.4.0; amd64-portbld-midnightbsd4; 2026-07-17 17:16:17 UTC; unix
