Package: utils
Version: 4.4.3
Priority: base
Title: The R Utils Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: R utility functions.
License: Part of R 4.4.3
Suggests: methods, xml2, commonmark, knitr, jsonlite
NeedsCompilation: yes
Built: R 4.4.3; aarch64-unknown-linux-gnu; 2025-02-28 11:55:54 UTC; unix
