Package: mgcv
Version: 1.9-4
Authors@R: person(given = "Simon",
                    family = "Wood",
                    role = c("aut", "cre"),
                    email = "simon.wood@r-project.org")
Title: Mixed GAM Computation Vehicle with Automatic Smoothness
        Estimation
Description: Generalized additive (mixed) models, some of their extensions and 
             other generalized ridge regression with multiple smoothing 
             parameter estimation by (Restricted) Marginal Likelihood, 
             Cross Validation and similar, or using iterated nested Laplace
	     approximation for fully Bayesian inference. See Wood (2025)
	     <doi:10.1146/annurev-statistics-112723-034249> for an overview.
	     Includes a gam() function, a wide variety of smoothers, 'JAGS'
	     support and distributions beyond the exponential family. 
Priority: recommended
Depends: R (>= 4.4.0), nlme (>= 3.1-64)
Imports: methods, stats, graphics, Matrix, splines, utils
Suggests: parallel, survival, MASS
LazyLoad: yes
ByteCompile: yes
License: GPL (>= 2)
NeedsCompilation: yes
Packaged: 2025-11-06 12:01:15 UTC; sw283
Author: Simon Wood [aut, cre]
Maintainer: Simon Wood <simon.wood@r-project.org>
Repository: CRAN
Date/Publication: 2025-11-07 16:30:02 UTC
Built: R 4.6.1; aarch64-unknown-linux-gnu; 2026-06-24 11:23:42 UTC; unix
