Package: tools
Version: 4.6.0
Priority: base
Title: Tools for Package Development
Author: R Core Team
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: Tools for package development, administration and documentation.
License: Part of R 4.6.0
Suggests: codetools, methods, xml2, curl, commonmark, knitr, xfun,
        mathjaxr, V8, bibtex
NeedsCompilation: yes
Built: R 4.6.0; armv6--netbsdelf-eabihf; 2026-08-21 19:00:08 UTC; unix
