Package: datasets
Version: 4.6.1
Priority: base
Title: The R Datasets Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: Base R datasets.
License: Part of R 4.6.1
Encoding: UTF-8
Built: R 4.6.1; ; 2026-06-24 12:34:19 UTC; unix
