Package: utils
Version: 4.6.0
Priority: base
Title: The R Utils Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: R utility functions.
License: Part of R 4.6.0
Suggests: methods, xml2, commonmark, knitr, jsonlite
NeedsCompilation: yes
Encoding: UTF-8
Built: R 4.6.0; armv6--netbsdelf-eabihf; 2026-08-21 19:03:34 UTC; unix
