Package: utils
Version: 4.6.1
Priority: base
Title: The R Utils Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: R utility functions.
License: Part of R 4.6.1
Suggests: methods, xml2, commonmark, knitr, jsonlite
NeedsCompilation: yes
Encoding: UTF-8
Built: R 4.6.1; aarch64-unknown-linux-gnu; 2026-06-24 11:15:31 UTC; unix
