Package: methods
Version: 4.5.3
Priority: base
Imports: utils, stats
Title: Formal Methods and Classes
Author: R Core Team
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: Formally defined methods and classes for R objects,
  plus other programming tools, as described in the reference.
References: John M. Chambers (2008) ``Software for Data Analysis:
        Programming with R''; Springer NY.
License: Part of R 4.5.3
Suggests: codetools
NeedsCompilation: yes
Built: R 4.5.3; x86_64-unknown-openbsd7.9; 2026-06-21 11:06:48 UTC; unix
