Package: parallel
Version: 4.6.1
Priority: base
Title: Support for Parallel Computation in R
Author: R Core Team
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: Support for parallel computation, including by forking
   (taken from package multicore), by sockets (taken from package snow)
   and random-number generation.
License: Part of R 4.6.1
Imports: tools, compiler
Suggests: methods
Enhances: snow, Rmpi, mirai, parallelly
NeedsCompilation: yes
Encoding: UTF-8
Built: R 4.6.1; x86_64-pc-linux-gnu; 2026-07-01 19:48:58 UTC; unix
