Package: graphics
Version: 2.8.1
Priority: base
Title: The R Graphics Package
Author: R Development Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: R functions for base graphics
Imports: grDevices
License: GPL (>= 2)
Built: R 2.8.1; ; 2016-11-11 13:49:54; unix
