Package: grid
Version: 2.8.1
Priority: base
Title: The Grid Graphics Package
Author: Paul Murrell <paul@stat.auckland.ac.nz>
Maintainer: R Core Team <R-core@r-project.org>
Description: A rewrite of the graphics layout capabilities, plus some
        support for interaction
Imports: grDevices
Suggests: lattice
License: GPL (>= 2)
Built: R 2.8.1; x86_64-unknown-linux-gnu; 2016-11-11 13:50:17; unix
