Package: splines
Version: 2.8.1
Priority: base
Imports: graphics, stats
Title: Regression Spline Functions and Classes
Author: Douglas M. Bates <bates@stat.wisc.edu> and William N. Venables
        <Bill.Venables@csiro.au>
Maintainer: R Core Team <R-core@r-project.org>
Description: Regression spline functions and classes
License: GPL (>= 2)
Built: R 2.8.1; x86_64-unknown-linux-gnu; 2016-11-11 13:50:21; unix
