Package: Biostrings
Title: String objects representing biological sequences, and matching
        algorithms
Description: Memory efficient string containers, string matching
        algorithms, and other utilities, for fast manipulation of large
        biological sequences or sets of sequences.
Version: 2.16.9
Author: H. Pages, P. Aboyoun, R. Gentleman, and S. DebRoy
Maintainer: H. Pages <hpages@fhcrc.org>
biocViews: SequenceMatching, Genetics, Infrastructure
Depends: R (>= 2.8.0), methods, IRanges (>= 1.6.6)
Imports: graphics, methods, stats, utils, IRanges, Biobase
LinkingTo: IRanges
Enhances: Rmpi
Suggests: BSgenome (>= 1.13.14), BSgenome.Celegans.UCSC.ce2 (>=
        1.3.11), BSgenome.Dmelanogaster.UCSC.dm3 (>= 1.3.11),
        drosophila2probe, hgu95av2probe, GenomicFeatures (>= 0.1.3),
        hgu95av2cdf, affy, affydata (>= 1.11.5), RUnit
License: Artistic-2.0
LazyLoad: yes
Collate: 00datacache.R utils.R IUPAC_CODE_MAP.R AMINO_ACID_CODE.R
        GENETIC_CODE.R XStringCodec-class.R xsbasetype.R
        XString-class.R XStringSet-class.R XStringViews-class.R
        XStringViews-constructors.R MaskedXString-class.R
        XStringSetList-class.R xscat.R readFASTA.R XStringSet-io.R
        letter.R subXString.R letterFrequency.R
        dinucleotideFrequencyTest.R chartr.R reverse.R translate.R
        extractTranscripts.R toComplex.R replaceLetterAt.R
        injectHardMask.R misc.R SparseList-class.R MIndex-class.R
        lowlevel-matching.R match-utils.R matchPattern.R maskMotif.R
        matchPattern.BOC.R matchPattern.BOC2.R matchLRPatterns.R
        trimLRPatterns.R matchProbePair.R matchPWM.R
        XKeySortedData-class.R XKeySortedDataList-class.R WCP-class.R
        matchWCP.R findPalindromes.R PDict-class.R matchPDict.R
        XStringPartialMatches-class.R XStringQuality-class.R
        QualityScaledXStringSet.R InDel-class.R
        AlignedXStringSet-class.R PairwiseAlignedXStringSet-class.R
        PairwiseAlignedFixedSubject-class.R align-utils.R
        pmatchPattern.R pairwiseAlignment.R stringDist.R needwunsQS.R
        matchprobes.R debug.R test.R zzz.R
Built: R 2.8.1; x86_64-unknown-linux-gnu; 2016-11-11 14:08:23; unix
