Package: foreign
Priority: recommended
Version: 0.8-29
Date: 2008-08-07
Title: Read Data Stored by Minitab, S, SAS, SPSS, Stata, Systat, dBase,
        ...
Depends: R (>= 2.6.0), stats
Imports: methods, utils
Maintainer: R-core <R-core@r-project.org>
Author: R-core members, Saikat DebRoy <saikat@stat.wisc.edu>, Roger
        Bivand <Roger.Bivand@nhh.no> and others: see COPYRIGHTS file in
        the sources.
Description: Functions for reading and writing data stored by
        statistical packages such as Minitab, S, SAS, SPSS, Stata,
        Systat, ..., and for reading and writing .dbf (dBase) files.
LazyLoad: yes
License: GPL (>= 2)
Packaged: Thu Aug 7 16:41:12 2008; ripley
Built: R 2.8.1; x86_64-unknown-linux-gnu; 2016-11-11 13:51:54; unix
