Package: modreg
Version: 1.9.1
Priority: defunct-base
Title: Defunct Package for Modern Regression: Smoothing and Local Methods
Author: Brian Ripley and the R Core Team
Maintainer: R Core Team <R-core@r-project.org>
Description: A stub package -- contents merged into stats
License: GPL Version 2 or later.
Built: R 1.9.1; ; Sun Mar 20 06:12:01 MST 2005; unix
