Package: IRanges
Title: Infrastructure for manipulating intervals on sequences
Description: The package provides efficient low-level and highly
        reusable S4 classes for storing ranges of integers, RLE vectors
        (Run-Length Encoding), and, more generally, data that can be
        organized sequentially, as well as views on these sequences.
        Efficient list-like classes are also provided for storing big
        collections of instances of the basic classes. All classes in
        the package use consistent naming and share the same rich and
        consistent "Sequence API" as much as possible.
Version: 1.6.17
Author: H. Pages, P. Aboyoun and M. Lawrence
Maintainer: Biocore Team c/o BioC user list
        <bioconductor@stat.math.ethz.ch>
biocViews: Infrastructure, Genetics
Depends: R (>= 2.8.0), methods, utils, stats
Imports: methods, utils, stats
Suggests: RUnit
License: Artistic-2.0
ExtraLicenses: The following files in the 'src' directory are licensed
        for all use by Jim Kent, in a manner compatible with the
        Artistic 2.0 license: common.c/h, memalloc.c/h, localmem.c/h,
        hash.c/h, errabort.c/h, rbTree.c/h, dlist.c/h, errCatch.h
Collate: validity-utils.R utils.R strutils.R compact_bitvector.R
        endoapply.R runstat.R Annotated-class.R Sequence-class.R
        DataTable-class.R DataTable-stats.R AtomicList-class.R
        Ranges-class.R Ranges-comparison.R Ranges-utils.R
        IRanges-class.R IRanges-constructor.R IRanges-utils.R
        IRanges-setops.R IntervalTree-class.R IntervalTree-utils.R
        Grouping-class.R Views-class.R Rle-class.R RleViews-class.R
        RleViews-utils.R SharedVector-class.R SharedRaw-class.R
        SharedInteger-class.R SharedDouble-class.R XVector-class.R
        XRaw-class.R XInteger-class.R XDouble-class.R
        XVectorList-class.R XIntegerViews-class.R XIntegerViews-utils.R
        SimpleList-class.R CompressedList-class.R AtomicList-impl.R
        DataFrame-class.R DataFrame-utils.R DataFrameList-class.R
        DataFrameList-utils.R RangesList-class.R Binning-class.R
        GappedRanges-class.R RleViewsList-class.R RleViewsList-utils.R
        MaskCollection-class.R RangedData-class.R FilterRules-class.R
        RDApplyParams-class.R RangedData-utils.R RangesMatching-class.R
        RangesMatchingList-class.R Alignment-class.R
        RangedSelection-class.R read.Mask.R xvcopy.R reverse.R
        coverage.R test_IRanges_package.R updateObject.R debug.R zzz.R
Built: R 2.8.1; x86_64-unknown-linux-gnu; 2016-11-11 14:28:27; unix
