Package: lattice
Version: 0.17-17
Date: 2008/11/20
Priority: recommended
Title: Lattice Graphics
Author: Deepayan Sarkar <deepayan.sarkar@r-project.org>
Maintainer: Deepayan Sarkar <deepayan.sarkar@r-project.org>
Description: Implementation of Trellis Graphics. See ?Lattice for a
        brief introduction
Depends: R (>= 2.5.0)
Suggests: grid
Imports: grid, grDevices, graphics, stats, utils
Enhances: chron
LazyLoad: yes
LazyData: yes
License: GPL (>= 2)
Packaged: Fri Nov 21 16:10:31 2008; dsarkar
Built: R 2.8.1; x86_64-unknown-linux-gnu; 2016-11-11 13:52:01; unix
