Package: rpart
Priority: recommended
Version: 3.1-42
Date: 2008-10-21
DateNote: March 2002 version of rpart
Author: Terry M Therneau and Beth Atkinson <atkinson@mayo.edu>. R port
        by Brian Ripley.
Maintainer: Brian Ripley <ripley@stats.ox.ac.uk>
Description: Recursive partitioning and regression trees
Title: Recursive Partitioning
Depends: R (>= 2.7.0), graphics, stats, grDevices
Suggests: survival
License: GPL-2 | file LICENCE
LazyLoad: yes
LazyData: yes
URL:
        http://mayoresearch.mayo.edu/mayo/research/biostat/splusfunctions.cfm
Packaged: Mon Oct 20 07:00:52 2008; ripley
Built: R 2.8.1; x86_64-unknown-linux-gnu; 2016-11-11 13:52:38; unix
