Package: rcompgen
Version: 0.1-17
Date: 2007/10/15
Priority: recommended
Title: Completion generator for R
Author: Deepayan Sarkar <deepayan.sarkar@r-project.org>
Maintainer: Deepayan Sarkar <deepayan.sarkar@r-project.org>
Description: Generates potential completions for a given command
        snippet based on circumstantial evidence
Depends: R (>= 2.5.0)
Imports: utils, methods
LazyLoad: yes
License: GPL (>= 2)
Packaged: Mon Oct 15 18:04:01 2007; dsarkar
Built: R 2.6.2; ; 2008-03-15 21:08:30; unix
