Package: KernSmooth
Priority: recommended
Version: 2.22-22
Date: 2008-01-26
Title: Functions for kernel smoothing for Wand & Jones (1995)
Author: S original by Matt Wand. R port by Brian Ripley
        <ripley@stats.ox.ac.uk>.
Maintainer: Brian Ripley <ripley@stats.ox.ac.uk>
Depends: R (>= 2.5.0), stats
Suggests: MASS
Description: functions for kernel smoothing (and density estimation)
        corresponding to the book: Wand, M.P. and Jones, M.C. (1995)
        "Kernel Smoothing".
License: Unlimited
Packaged: Sat Jan 26 10:35:47 2008; ripley
Built: R 2.8.1; x86_64-unknown-linux-gnu; 2016-11-11 13:51:58; unix
