Package: Biobase
Title: Biobase: Base functions for Bioconductor
Version: 2.8.0
Author: R. Gentleman, V. Carey, M. Morgan, S. Falcon
Description: Functions that are needed by many other packages or which
        replace R functions.
Suggests: tools, tkWidgets, ALL
Depends: R (>= 2.7.0), methods, utils
Maintainer: Biocore Team c/o BioC user list
        <bioconductor@stat.math.ethz.ch>
License: Artistic-2.0
Collate: tools.R strings.R environment.R vignettes.R packages.R
        AllGeneric.R VersionsClass.R VersionedClasses.R
        methods-VersionsNull.R methods-VersionedClass.R DataClasses.R
        methods-aggregator.R methods-ANY.R methods-container.R
        methods-data.frame.R methods-matrix.R methods-phenoData.R
        methods-MIAME.R methods-annotatedDataset.R methods-AssayData.R
        methods-AnnotatedDataFrame.R methods-eSet.R
        methods-ExpressionSet.R methods-MultiSet.R methods-SnpSet.R
        methods-NChannelSet.R anyMissing.R methods-exprSet.R
        rowOp-methods.R updateObject.R updateObjectTo.R
        methods-ScalarObject.R zzz.R
LazyLoad: yes
biocViews: Infrastructure, Bioinformatics
Built: R 2.8.1; x86_64-unknown-linux-gnu; 2016-11-11 14:08:11; unix
