Package: cluster
Version: 1.11.11
Date: 2008-06-14
Priority: recommended
Author: Martin Maechler, based on S original by Peter Rousseeuw
        <rousse@uia.ua.ac.be>, Anja.Struyf@uia.ua.ac.be and
        Mia.Hubert@uia.ua.ac.be, and initial R port by
        Kurt.Hornik@R-project.org
Maintainer: Martin Maechler <maechler@stat.math.ethz.ch>
Title: Cluster Analysis Extended Rousseeuw et al.
Description: Cluster Analysis, extended original from Peter Rousseeuw,
        Anja Struyf and Mia Hubert.
Depends: R (>= 2.5.0), stats, graphics, utils
LazyLoad: yes
LazyData: yes
License: GPL (>= 2)
Packaged: Mon Jun 16 09:54:08 2008; maechler
Built: R 2.8.1; x86_64-unknown-linux-gnu; 2016-11-11 13:51:43; unix
