Package: exS4noNS
Title: Example with S4 Methods and Classes, but no NameSpace
Type: Package
Depends: methods
Version: 1.0
Date: 2008-07-28
Author: Anonymous R-core
Maintainer: R Core <R-core@almost.r-project.org>
Description: Example package with S4 methods and classes, but
 *no* name space; used for regression testing the correct working of
 tools::codoc(), undoc() etc, but also S4 in connection with other
 packages.
License: GPL (>= 2)
LazyLoad: yes

