Package: lqs
Version: 1.9.1
Priority: defunct-base
Title: Resistant Regression and Covariance Estimation
Author: Brian Ripley.
Maintainer: R Core Team <R-core@r-project.org>
Description: A stub package -- contents merged into MASS
License: GPL Version 2 or later.
Built: R 1.9.1; ; Sun Mar 20 06:12:00 MST 2005; unix
