Package: nlme
Version: 3.1-89
Date: 2008-06-07
Priority: recommended
Title: Linear and Nonlinear Mixed Effects Models
Author: Jose Pinheiro <Jose.Pinheiro@pharma.novartis.com>, Douglas
        Bates <bates@stat.wisc.edu>, Saikat DebRoy
        <saikat@stat.wisc.edu>, Deepayan Sarkar
        <Deepayan.Sarkar@R-project.org>, the R Core team.
Maintainer: R-core <R-core@R-project.org>
Description: Fit and compare Gaussian linear and nonlinear
        mixed-effects models.
Depends: graphics, stats, R (>= 2.4.0)
Imports: lattice
LazyLoad: yes
LazyData: yes
License: GPL (>=2)
Packaged: Sat Jun 7 14:48:00 2008; ripley
Built: R 2.8.1; x86_64-unknown-linux-gnu; 2016-11-11 13:52:08; unix
