Package: ctest
Version: 1.9.1
Priority: defunct-base
Title: Defunct Package for Classical Tests
Author: Kurt Hornik <Kurt.Hornik@ci.tuwien.ac.at>, with major
  contributions by Peter Dalgaard <p.dalgaard@kubism.ku.dk> and
  Torsten Hothorn <Torsten.Hothorn@rzmail.uni-erlangen.de>.
Maintainer: R Core Team <R-core@r-project.org>
Description: A stub package -- contents merged into stats
License: GPL
Built: R 1.9.1; ; Sun Mar 20 06:11:59 MST 2005; unix
