Package: codetools
Version: 0.2-1
Priority: recommended
Author: Luke Tierney <luke@stat.uiowa.edu>
Description: Code analysis tools for R
Title: Code Analysis Tools for R
Depends: R (>= 2.1)
Maintainer: Luke Tierney <luke@stat.uiowa.edu>
License: GPL
Packaged: Wed Apr 2 07:33:10 2008; luke
Built: R 2.8.1; ; 2016-11-11 13:51:50; unix
