Package: base
Version: 2.8.1
Priority: base
Title: The R Base Package
Author: R Development Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: Base R functions
License: GPL (>= 2)
Built: R 2.8.1; ; Fri Nov 11 13:49:46 EET 2016; unix
