Package: mgcv
Version: 1.4-1.1
Author: Simon Wood <simon.wood@r-project.org>
Maintainer: Simon Wood <simon.wood@r-project.org>
Title: GAMs with GCV smoothness estimation and GAMMs by REML/PQL
Description: Routines for GAMs and other generalized ridge regression
        with multiple smoothing parameter selection by GCV or UBRE/AIC.
        Also GAMMs by REML or PQL. Includes a gam() function.
Priority: recommended
Depends: R (>= 2.3.0)
Imports: graphics, stats
Suggests: nlme (>= 3.1-64), splines
LazyLoad: yes
License: GPL (>=2)
Packaged: Sat Dec 13 11:55:01 2008; hornik
Built: R 2.8.1; x86_64-unknown-linux-gnu; 2016-11-11 13:52:25; unix
