Package: stepfun
Version: 1.9.1
Priority: defunct-base
Title: Defunct Package for Step Functions, incl. Empirical Distributions
Author: R Development Core Team
Maintainer: R Core Team <R-core@r-project.org>
Description: A stub package -- contents merged into stats
License: GPL Version 2 or later.
Built: R 1.9.1; ; Sun Mar 20 06:12:03 MST 2005; unix
