Package: mle
Title: Defunct package for maximum likelihood estimation
Version: 1.9.1
Priority: defunct-base
Author: Peter Dalgaard
Description: A stub package -- contents merged into stats
Maintainer: R Core Team <R-core@r-project.org>
License: GPL
Built: R 1.9.1; ; Sun Mar 20 06:12:01 MST 2005; unix
