Package: tcltk
Version: 2.8.1
Priority: base
Title: Tcl/Tk Interface
Author: R Development Core Team
Maintainer: R Core Team <R-core@r-project.org>
Description: Interface and language bindings to Tcl/Tk GUI elements
License: GPL (>= 2)
Built: R 2.8.1; x86_64-unknown-linux-gnu; 2016-11-11 13:50:22; unix
