Package: tools
Version: 2.8.1
Priority: base
Title: Tools for Package Development
Author: Kurt Hornik and Friedrich Leisch
Maintainer: R Core Team <R-core@r-project.org>
Description: Tools for package development, administration and
        documentation
License: GPL (>= 2)
Built: R 2.8.1; x86_64-unknown-linux-gnu; 2016-11-11 13:49:49; unix
