Bundle: VR
Priority: recommended
Contains: MASS class nnet spatial
Version: 7.2-45
Date: 2008-12-07
Depends: R (>= 2.4.0), grDevices, graphics, stats, utils
Suggests: lattice, nlme, survival
Author: S original by Venables & Ripley. R port by Brian Ripley
        <ripley@stats.ox.ac.uk>, following earlier work by Kurt Hornik
        and Albrecht Gebhardt.
Maintainer: Brian Ripley <ripley@stats.ox.ac.uk>
BundleDescription: Functions and datasets to support Venables and
        Ripley, 'Modern Applied Statistics with S' (4th edition).
License: GPL-2 | GPL-3
URL: http://www.stats.ox.ac.uk/pub/MASS4/
Packaged: Tue Dec 9 11:11:17 2008; ripley
Package: nnet
Description: Software for feed-forward neural networks with a single
        hidden layer, and for multinomial log-linear models.
Title: Feed-forward Neural Networks and Multinomial Log-Linear Models
LazyLoad: yes
Built: R 2.8.1; x86_64-unknown-linux-gnu; 2016-11-11 13:51:34; unix
