Package: stats4
Title: Statistical Functions using S4 Classes
Version: 2.8.1
Priority: base
Author: R Development Core Team and contributors worldwide
Description: Statistical Functions using S4 classes
Maintainer: R Core Team <R-core@r-project.org>
Depends: methods, graphics, stats
LazyLoad: yes
License: GPL (>= 2)
Built: R 2.8.1; ; 2016-11-11 13:50:21; unix
