useDynLib(MASS, .registration = TRUE)

# functions
export(addterm, area, as.fractions, bandwidth.nrd, bcv, boxcox,
       con2tr, contr.sdif, corresp, cov.trob, denumerate, dose.p,
       dropterm, enlist, eqscplot, fbeta, fitdistr, fractions,
       frequency.polygon, gamma.dispersion, gamma.shape, ginv,
       glm.convert, glm.nb, glmmPQL, hist.FD, hist.scott, huber,
       hubers, is.fractions, isoMDS, kde2d, lda, ldahist, lm.gls,
       lm.ridge, loglm, loglm1, logtrans, mca, mvrnorm,
       nclass.freq, neg.bin, negative.binomial, negexp.SSival, Null,
       parcoord, polr, psi.bisquare, psi.hampel, psi.huber, qda,
       rational, renumerate, rlm, rms.curv, rnegbin, sammon, select,
       Shepard, stdres, stepAIC, studres, theta.md, theta.ml,
       theta.mm, truehist, ucv, width.SJ, write.matrix)

# formerly in package lqs
export(cov.mcd, cov.mve, cov.rob, lmsreg, lqs, lqs.formula, ltsreg)

# formerly needed for Recall
# export(denumerate.formula, renumerate.formula)

# documented, registered but unexported methods
# export(addterm.default, addterm.glm, addterm.lm,
#        anova.loglm, anova.negbin,
#        boxcox.default, boxcox.formula, boxcox.lm,
#        confint.glm, confint.nls,
#        dropterm.default, dropterm.glm, dropterm.lm,
#        gamma.shape.glm,
#        loglm1.data.frame, loglm1.default, loglm1.xtabs,
#        logtrans.default, logtrans.formula, logtrans.lm,
#        pairs.lda, pairs.profile, plot.lda, plot.mca,
#        plot.profile, plot.ridgelm, predict.lda, predict.mca,
#        predict.qda, print.abbrev, print.Anova,
#        profile.glm, select.ridgelm,
#        summary.loglm, summary.negbin, summary.rlm)

importFrom(grDevices, dev.flush, dev.hold, nclass.FD, nclass.scott, palette)
importFrom(graphics, abline, axis, box, frame, hist, lines, matplot, mtext,
           pairs, par, plot, points, rect, segments, symbols, text)
importFrom(stats, add1, anova, biplot, coef, confint, drop1,
	   extractAIC, family, fitted, logLik, model.frame, nobs,
	   predict, profile, residuals, se.contrast, simulate, terms,
	   update, vcov,
           ## added in 7.3-42
           .checkMFClasses, .getXlevels, IQR, add.scope, aov, approx,
           as.formula, binomial, cmdscale, contr.helmert, dbeta, dcauchy,
           dchisq, delete.response, density, deviance, dexp, df,
           dgamma, dgeom, dist, dlnorm, dlogis, dnbinom, dnorm, dpois,
           drop.scope, dt, dweibull, factor.scope, formula, glm,
           glm.control, glm.fit, is.empty.model, lm, lm.fit,
           lm.influence, lm.wfit, loglin, lsfit, mad, mahalanobis,
           make.link, median, model.extract, model.matrix,
           model.offset, model.response, model.weights, na.pass,
           napredict, naprint, naresid, optim, optimize, pcauchy,
           pchisq, pf, plogis, pnorm, qchisq, qf, qnorm, qt, quantile,
           rcauchy, rexp, rgamma, rlogis, rnorm, rpois, runif, sd,
           spline, summary.glm, uniroot, update.formula, var,
           ## added in 7.3-58
           drop.terms, reformulate)

S3method("[", fractions)
S3method("[<-", fractions)
S3method(addterm, default)
S3method(addterm, glm)
S3method(addterm, lm)
S3method(addterm, mlm)
S3method(addterm, negbin)
S3method(addterm, survreg)
S3method(anova, glmmPQL)
S3method(anova, loglm)
S3method(anova, negbin)
S3method(anova, polr)
S3method(as.character, fractions)
S3method(biplot, correspondence)
S3method(boxcox, default)
S3method(boxcox, formula)
S3method(boxcox, lm)
S3method(coef, fitdistr)
S3method(coef, lda)
S3method(coef, loglm)
S3method(coef, ridgelm)
## Taken by PD for R 4.4.0
#S3method(confint, glm)
#S3method(confint, nls)
S3method(confint, polr)
## Taken by PD for R 4.4.0
#S3method(confint, profile.glm)
#S3method(confint, profile.nls)
S3method(confint, profile.polr)
S3method(corresp, xtabs)
S3method(corresp, data.frame)
S3method(corresp, default)
S3method(corresp, factor)
S3method(corresp, formula)
S3method(corresp, matrix)
S3method(denumerate, formula)
S3method(dropterm, default)
S3method(dropterm, glm)
S3method(dropterm, lm)
S3method(dropterm, mlm)
S3method(dropterm, negbin)
S3method(dropterm, survreg)
#S3method(extractAIC, gls)
#S3method(extractAIC, lme)
S3method(extractAIC, loglm)
S3method(extractAIC, polr)
S3method(family, negbin)
S3method(fitted, loglm)
S3method(formula, glmmPQL)
S3method(gamma.shape, glm)
S3method(lda, data.frame)
S3method(lda, default)
S3method(lda, formula)
S3method(lda, matrix)
S3method(loglm1, data.frame)
S3method(loglm1, default)
S3method(loglm1, xtabs)
S3method(logLik, fitdistr)
S3method(logLik, glmmPQL)
S3method(logLik, negbin)
S3method(logLik, polr)
S3method(logtrans, default)
S3method(logtrans, formula)
S3method(logtrans, lm)
S3method(lqs, default)
S3method(lqs, formula)
S3method(Math, fractions)
S3method(model.frame, lda)
S3method(model.frame, polr)
S3method(model.frame, qda)
S3method(nobs, loglm)
S3method(nobs, polr)
S3method(Ops, fractions)
S3method(pairs, lda)
## Taken by PD for R 4.4.0
#S3method(pairs, profile)
S3method(plot, correspondence)
S3method(plot, lda)
S3method(plot, mca)
## Taken by PD for R 4.4.0
#S3method(plot, profile)
S3method(plot, ridgelm)
S3method(predict, glmmPQL)
S3method(predict, lda)
S3method(predict, lqs)
S3method(predict, mca)
S3method(predict, polr)
S3method(predict, qda)
S3method(predict, rlm)
S3method(print, abbrev)
S3method(print, Anova)
S3method(print, anova.loglm)
S3method(print, correspondence)
S3method(print, fitdistr)
S3method(print, fractions)
S3method(print, gamma.shape)
S3method(print, glm.dose)
S3method(print, lda)
S3method(print, loglm)
S3method(print, lqs)
S3method(print, mca)
S3method(print, polr)
S3method(print, qda)
S3method(print, ridgelm)
S3method(print, rlm)
S3method(print, rms.curv)
S3method(print, summary.loglm)
S3method(print, summary.negbin)
S3method(print, summary.polr)
S3method(print, summary.rlm)
#S3method(profile, glm)
S3method(profile, polr)
S3method(qda, data.frame)
S3method(qda, default)
S3method(qda, formula)
S3method(qda, matrix)
S3method(renumerate, formula)
S3method(residuals, loglm)
S3method(rlm, default)
S3method(rlm, formula)
S3method(se.contrast, rlm)
S3method(select, ridgelm)
S3method(simulate, negbin)
S3method(simulate, polr)
S3method(Summary, fractions)
S3method(summary, loglm)
S3method(summary, negbin)
S3method(summary, polr)
S3method(summary, rlm)
S3method(t, fractions)
#S3method(terms, gls)
#S3method(terms, lme)
S3method(update, loglm)
S3method(vcov, fitdistr)
S3method(vcov, negbin)
S3method(vcov, polr)
S3method(vcov, rlm)
