useDynLib(spatial, .registration = TRUE)

export(anova.trls, anovalist.trls, correlogram, expcov, gaucov, Kaver,
       Kenvl, Kfn, plot.trls, ppgetregion, ppinit, pplik, ppregion,
       predict.trls, prmat, Psim, semat, sphercov, SSI, Strauss,
       surf.gls, surf.ls, trls.influence, trmat, variogram)

importFrom(graphics, abline, plot, points, symbols)
importFrom(stats, anova, deviance, df.residual, extractAIC, fitted,
           pf, predict, quantile, residuals, var)

S3method(anova,trls)
S3method(deviance,trls)
S3method(df.residual,trls)
S3method(extractAIC,trls)
S3method(fitted,trls)
S3method(plot,trls)
S3method(predict,trls)
S3method(residuals,trls)
S3method(summary,trls)
