exportPattern("^tcl[[:alnum:]]*($|<-)") # skip S3 methods
exportPattern("^tk")
exportPattern("^ttk")
exportPattern("^.Tcl")
exportPattern("^.Tk")
export(addTclPath, as.tclObj, is.tclObj, is.tkwin)
export(tclfile.dir, tclfile.tail)
export(getTkProgressBar, setTkProgressBar)

##S3method("$", tclvar)
##S3method("$<-", tclvar)
S3method(as.character, tclObj)
S3method(as.character, tclVar)
S3method(as.double, tclObj)
S3method(as.integer, tclObj)
S3method(as.logical, tclObj)
S3method(as.raw, tclObj)
S3method(print, tclObj)
S3method("[[", tclArray)
S3method("[[<-", tclArray)
S3method("$", tclArray)
S3method("$<-", tclArray)
S3method("names", tclArray)
S3method("names<-", tclArray)
S3method("length", tclArray)
S3method("length<-", tclArray)
S3method(tclObj, tclVar)
S3method("tclObj<-", tclVar)
S3method("tclvalue", default)
S3method("tclvalue", tclObj)
S3method("tclvalue", tclVar)
S3method("tclvalue<-", default)
S3method("tclvalue<-", tclVar)

S3method(close, tkProgressBar)

