export(walkCode,makeCodeWalker)

export(showTree)

export(makeConstantFolder,constantFold)
export(isConstantValue)

export(getAssignedVar)
export(makeLocalsCollector,collectLocals)
export(findLocals,findLocalsList)
export(findFuncLocals)

export(flattenAssignment)

export(makeUsageCollector,collectUsage)
export(findGlobals)

export(checkUsage, checkUsageEnv, checkUsagePackage)
