useDynLib(rpart, .registration = TRUE, .fixes = "C_")

export(meanvar, na.rpart, path.rpart, plotcp, post, printcp, prune,
       prune.rpart, rpart, rpart.control, rsq.rpart, snip.rpart, xpred.rpart)
export(rpart.exp)  # needed for one of the tests

importFrom(grDevices, col2rgb, dev.cur, dev.off, postscript)
importFrom(graphics, abline, axis, box, identify, legend, lines, mtext,
           par, plot, polygon, segments, text, title)
import(stats)

S3method(labels, rpart)
S3method(meanvar, rpart)
S3method(model.frame, rpart)
S3method(plot, rpart)
S3method(post, rpart)
S3method(predict, rpart)
S3method(print, rpart)
S3method(prune, rpart)
S3method(residuals, rpart)
S3method(summary, rpart)
S3method(text, rpart)

