useDynLib(nnet, .registration = TRUE)

export(class.ind, multinom, nnet, nnetHess, which.is.max)

# match.call problem
export(nnet.default, nnet.formula)

#importFrom(stats, add1, anova, coef, drop1, extractAIC, model.frame, predict, vcov)
import(stats)
importFrom(utils, flush.console)

S3method(add1, multinom)
S3method(anova, multinom)
S3method(coef, multinom)
S3method(confint, multinom)
S3method(drop1, multinom)
S3method(extractAIC, multinom)
S3method(logLik, multinom)
S3method(model.frame, multinom)
S3method(predict, multinom)
S3method(print, multinom)
S3method(print, summary.multinom)
S3method(summary, multinom)
S3method(vcov, multinom)

S3method(coef, nnet)
S3method(predict, nnet)
S3method(print, nnet)
S3method(print, summary.nnet)
# S3method(residuals, nnet)
S3method(summary, nnet)

S3method(nnet, default)
S3method(nnet, formula)
