useDynLib(foreign, .registration = TRUE)

export(data.restore, lookup.xport, read.arff, read.dbf, read.dta,
       read.epiinfo, read.mtp, read.octave, read.S, read.spss, read.ssd,
       read.systat, read.xport, write.arff, write.dbf, write.dta,
       write.foreign)

# not exported: SModeNames writeForeignStata writeForeignSPSS

importFrom(utils, download.file, read.table, write.table)
importFrom(stats, na.omit)
