export(abc.ci, boot, boot.array, boot.ci, censboot, control, corr,
       cum3, cv.glm, EEF.profile, EL.profile, empinf, envelope,
       exp.tilt, freq.array, glm.diag, glm.diag.plots, imp.moments,
       imp.prob, imp.quantile, imp.weights, inv.logit,
       jack.after.boot, k3.linear, lik.CI, linear.approx, logit,
       nested.corr, norm.ci, saddle, saddle.distn, simplex,
       smooth.f, tilt.boot, tsboot, var.linear)

# documented, registered but not exported
# export(lines.saddle.distn, plot.boot, print.boot, print.bootci, print.simplex)

importFrom(graphics, abline, hist, identify, layout, lines, par, plot, text)
import(stats)

S3method(c, boot)
S3method(lines, saddle.distn)
S3method(plot, boot)
S3method(print, boot)
S3method(print, bootci)
S3method(print, saddle.distn)
S3method(print, simplex)
