useDynLib(graphics,
          C_contour, C_contourDef, C_filledcontour, C_image,
          C_persp, C_abline, C_arrows, C_axis, C_box, C_clip,
          C_convertX, C_convertY, C_dend, C_dendwindow, C_erase,
          C_layout, C_mtext, C_par, C_path, C_plotXY, C_plot_window,
          C_polygon, C_raster, C_rect, C_segments, C_strHeight,
          C_strWidth, C_symbols, C_text, C_title, C_xspline,
          C_plot_new, C_locator, C_identify, C_StemLeaf, C_BinCount,
          RunregisterBase)

import(grDevices)
# not importFrom(stats, ..)  as stats imports us

export(abline, arrows, assocplot, axTicks, Axis, axis, axis.Date,
       axis.POSIXct, barplot, box, boxplot, bxp, cdplot, clip,
       close.screen, co.intervals, contour, coplot, curve, dotchart,
       erase.screen, filled.contour, fourfoldplot, frame, grconvertX,
       grconvertY, grid, hist, identify, image, layout, layout.show,
       lcm, legend, lines, locator, matlines, matplot, matpoints,
       mosaicplot, mtext, pairs, panel.smooth, par, persp, pie, 
       plot,
       plot.design, plot.function, plot.new, plot.window, plot.xy,
       points, polygon, polypath, rasterImage, rect, rug, screen,
       segments, smoothScatter, spineplot, split.screen, stars, stem,
       strheight, stripchart, strwidth, sunflowerplot, symbols, text,
       title, xinch, xspline, xyinch, yinch, .filled.contour)

# leave most of the default methods visible as people seem to call them.
export(barplot.default, boxplot.default, boxplot.matrix,
       contour.default, hist.default, image.default, lines.default,
       pairs.default, plot.default, points.default, text.default)

S3method(Axis, default)
S3method(Axis, Date)
S3method(Axis, POSIXt)
S3method(Axis, table)
S3method(barplot, default)
S3method(barplot, formula)
S3method(boxplot, default)
S3method(boxplot, formula)
S3method(boxplot, matrix)
S3method(cdplot, default)
S3method(cdplot, formula)
S3method(contour, default)
S3method(hist, Date)
S3method(hist, default)
S3method(hist, POSIXt)
S3method(identify, default)
S3method(image, default)
S3method(lines, default)
S3method(lines, formula)
S3method(lines, histogram)
S3method(lines, table)
S3method(mosaicplot, default)
S3method(mosaicplot, formula)
S3method(pairs, default)
S3method(pairs, formula)
S3method(persp, default)
S3method(plot, data.frame)
S3method(plot, default)
S3method(plot, factor)
S3method(plot, formula)
S3method(plot, "function")
S3method(plot, histogram)
S3method(plot, raster)
S3method(plot, table)
S3method(points, default)
S3method(points, formula)
S3method(points, table)
S3method(spineplot, default)
S3method(spineplot, formula)
S3method(stripchart, default)
S3method(stripchart, formula)
S3method(sunflowerplot, default)
S3method(sunflowerplot, formula)
S3method(text, default)
S3method(text, formula)
