export(mle) # exporting methods implicitly exports the generics

importFrom("grDevices", dev.flush, dev.hold)
importFrom("graphics", abline, lines, par)
importFrom("methods", new, show, slotNames,
           setClass, setGeneric, setMethod, representation, signature)
importFrom("stats", approx, optim, pchisq, predict, qchisq, qnorm, spline)
## For these, we define methods or (AIC, BIC, nobs) an implicit generic:
importFrom("stats",
	   AIC, BIC, coef, confint, logLik,
	   na.omit, nobs, profile, update, vcov)

## S4
exportClasses(mle, profile.mle, summary.mle)
## All methods for imported generics:
exportMethods(coef, confint, logLik, nobs, plot, profile, summary, show, update, vcov)
## implicit generics do not have any methods here
export(AIC, BIC)
