useDynLib(tools)

export("Rdindex",
       "checkDocFiles", "checkDocStyle", "checkFF", "checkMD5sums",
       "checkReplaceFuns", "checkS3methods", "checkTnF", "checkVignettes",
       "codoc", "codocClasses", "codocData", "md5sum", "undoc",
       "file_path_as_absolute", "file_path_sans_ext", "file_test",
       "list_files_with_exts", "list_files_with_type", "delimMatch",
       "texi2dvi", "buildVignettes", "pkgVignettes", "pkgDepends",
       "getDepList", "installFoundDepends", "vignetteDepends")

export("Rd_db", "Rd_parse", "package.dependencies", "read.00Index")

## <FIXME>
## Remove in 2.1.
export("filePathAsAbsolute", "filePathSansExt", "fileTest",
       "listFilesWithExts", "listFilesWithType")
## </FIXME>

S3method("print", "checkDocFiles")
S3method("print", "checkDocStyle")
S3method("print", "checkFF")
S3method("print", "checkReplaceFuns")
S3method("print", "checkS3methods")
S3method("print", "checkTnF")
S3method("print", "checkVignettes")
S3method("print", "codoc")
S3method("print", "codocClasses")
S3method("print", "codocData")
S3method("print", "undoc")

S3method("print", "check_demo_index")
S3method("print", "check_package_depends")
S3method("print", "check_package_description")
S3method("print", "check_vignette_index")
S3method("print", "check_Rd_files_in_Rd_db")
