import(grDevices)

export(abline, arrows, assocplot, axTicks, Axis, axis, axis.Date,
       axis.POSIXct, barplot, barplot, box, boxplot, bxp, "cdplot",
       close.screen, co.intervals, contour, coplot, curve, dotchart,
       erase.screen, filled.contour, fourfoldplot, frame, grid, hist,
       hist.default, identify, image, layout, layout.show, lcm, legend,
       lines, lines.default, locator, matlines, matplot, matpoints,
       mosaicplot, mtext, pairs, panel.smooth, par, persp, pie, plot,
       plot.design, plot.new, plot.window, plot.xy, points, polygon,
       rect, rug, screen, segments, "spineplot", split.screen, stars,
       stem, strheight, stripchart, strwidth, sunflowerplot, symbols,
       text, title, xinch, xyinch, yinch)

# leave most of the default methods visible as people seem to call them.
export(barplot.default, boxplot.default, contour.default, image.default,
       pairs.default, plot.default, points.default, text.default)

#defunct
export(piechart)

S3method(Axis, default)
S3method(Axis, Date)
S3method(Axis, POSIXct)
S3method(Axis, POSIXlt)
S3method(barplot, default)
S3method(boxplot, default)
S3method(boxplot, formula)
S3method("cdplot", "default")
S3method("cdplot", "formula")
S3method(contour, default)
S3method(hist, Date)
S3method(hist, default)
S3method(hist, POSIXt)
S3method(identify, default)
S3method(image, default)
S3method(lines, default)
S3method(lines, formula)
S3method(lines, histogram)
S3method(mosaicplot, default)
S3method(mosaicplot, formula)
S3method(pairs, default)
S3method(pairs, formula)
S3method(persp, default)
S3method(plot, data.frame)
S3method(plot, Date)
S3method(plot, default)
S3method(plot, factor)
S3method(plot, formula)
S3method(plot, histogram)
S3method(plot, POSIXct)
S3method(plot, POSIXlt)
S3method(plot, table)
S3method(points, default)
S3method(points, formula)
S3method("spineplot", "default")
S3method("spineplot", "formula")
S3method(text, default)
