# Explicitly enumerate the symbols here that are in the registration information
# rather than use .registration as there are symbols that conflict with functions
# on Windows, i.e. savePlot.  devga and Quartz are also not listed
# here since they are conditional as is savePlot.  Calls to those are currently
# left in name, PACKAGE pairs.
# We can add additional useDynLib() directives within the conditional constructs
# below to add these and can use a different prefix.

useDynLib(grDevices, R_chull,
          PicTeX, PostScript, XFig, PDF, 
          Type1FontInUse, CIDFontInUse, R_GD_nullDevice)

export(Hershey, boxplot.stats, check.options, chull,
       CIDFont, 
       col2rgb, colors, colours, cm, colorRamp,
       colorRampPalette, contourLines,
       convertColor, colorConverter, colorspaces,
       cm.colors, dev.control, dev.copy, dev.copy2eps, dev.cur,
       dev.interactive, dev.list, dev.next, dev.off, dev.prev,
       dev.print, dev.set, embedFonts, extendrange,
       getGraphicsEvent, graphics.off, gray, grey,
       gray.colors, grey.colors, heat.colors, hsv, hcl, make.rgb, n2mfrow,
       nclass.Sturges, nclass.FD, nclass.scott,
       palette,
       pdf, pdfFonts,
       pictex,
       postscript, postscriptFont, postscriptFonts, 
       ps.options, rainbow, recordGraphics, recordPlot,
       replayPlot, rgb, rgb2hsv, terrain.colors, topo.colors, trans3d,
       Type1Font, 
       xfig, xy.coords, xyz.coords)
export(X11, x11, bitmap, dev2bitmap, jpeg, png)

S3method(print, recordedplot)
S3method(print, colorConverter)
S3method(print, RGBcolorConverter)

S3method("checkFont", "default")
S3method("checkFont", "Type1Font")
S3method("checkFont", "CIDFont")
S3method("matchEncoding", "Type1Font")
S3method("matchEncoding", "CIDFont")

if(tools:::.OStype() == "unix") {
export(X11Font, X11Fonts, quartz, quartzFont, quartzFonts)
}

if(tools:::.OStype() == "windows") {
export(bmp, bringToTop, savePlot, win.graph, win.metafile, win.print, windows,
       windowsFont, windowsFonts)

S3method("[", SavedPlots)
S3method(print, SavedPlots)
}
