useDynLib(methods) #>> fails for now : .registration = TRUE

importFrom(utils, find)


exportPattern("^\\.__C__")
exportPattern("^\\.__M__")
exportPattern("^\\.__T__")

export(.Last.lib)
export(.TraceWithMethods)
export(.untracedFunction)
export(.doTracePrint)
export(.valueClassTest)
export("@<-")
export(Arith)
export(Compare)
export(Complex)
export(Logic)
export(Math)
export(Math2)
export(MethodAddCoerce)
export(MethodsList)
export(MethodsListSelect)
export(Ops)
export(Quote)
export(SignatureMethod)
export(S3Class)
export("S3Class<-")
export(S3Part)
export("S3Part<-")
export(Summary)
export(addNextMethod)
export(allGenerics)
export(allNames)
export(as)
export("as<-")
export(asMethodDefinition)
export(assignClassDef)
export(assignMethodsMetaData)
export(balanceMethodsList)
export("body<-")
export(cacheGenericsMetaData)
export(cacheMetaData)
export(cacheMethod)
export(callGeneric)
export(callNextMethod)
export(canCoerce)
export(checkSlotAssignment)
export(classMetaName)
export(classesToAM)
export(coerce)
export("coerce<-")
export(completeClassDefinition)
export(completeExtends)
export(completeSubclasses)
export(conformMethod)
export(defaultDumpName)
export(defaultPrototype)
export(doPrimitiveMethod)
export(dumpMethod)
export(dumpMethods)
export(el)
export("el<-")
export(elNamed)
export("elNamed<-")
export(empty.dump)
export(emptyMethodsList)
export(evalSource)
export(existsFunction)
export(existsMethod)
export(extends)
export(finalDefaultMethod)
export(findClass)
export(findFunction)
export(findMethod)
export(findMethods)
export(findMethodSignatures)
export(findUnique)
export(fixPre1.8)
export(formalArgs)
export(functionBody)
export("functionBody<-")
export(generic.skeleton)
export(getAccess)
export(getAllMethods)
export(getAllSuperClasses)
export(getClass)
export(getClassDef)
export(getClassName)
export(getClassPackage)
export(getClasses)
export(getDataPart)
export(getExtends)
export(getFunction)
export(getGeneric)
export(getGenerics)
export(getGroup)
export(getGroupMembers)
export(getMethod)
export(getMethods)
export(getMethodsForDispatch)
export(getMethodsMetaData)
export(getPackageName)
export(getProperties)
export(getPrototype)
export(getSlots)
export(getSubclasses)
export(getValidity)
export(getVirtual)
export(hasArg)
export(hasMethod)
export(hasMethods)
export(inheritedSlotNames)
export(initialize)
export(insertMethod)
export(insertSource)
export(is)
export(isClass)
export(isClassDef)
export(isClassUnion)
export(isGeneric)
export(isGrammarSymbol)
export(isGroup)
export(isSealedClass)
export(isSealedMethod)
export(isVirtualClass)
export(isXS3Class)
export(languageEl)
export("languageEl<-")
export(linearizeMlist)
export(listFromMethods)
export(listFromMlist)
export(loadMethod)
export(makeClassRepresentation)
export(makeExtends)
export(makeGeneric)
export(makeMethodsList)
export(makePrototypeFromClassDef)
export(makeStandardGeneric)
export(matchSignature)
export(mergeMethods)
export(metaNameUndo)
export(methodSignatureMatrix)
export(methodsPackageMetaName)
export(method.skeleton)
export(missingArg)
export(mlistMetaName)
export(new)
export(newBasic)
export(newClassRepresentation)
export(selectSuperClasses, .selectSuperClasses)
export(newEmptyObject)
export(packageSlot)
export("packageSlot<-")
export(possibleExtends)
export(promptClass)
export(promptMethods)
export(prototype)
export(reconcilePropertiesAndPrototype)
export(rematchDefinition)
export(removeClass)
export(removeGeneric)
export(removeMethod)
export(removeMethods)
export(removeMethodsObject)
export(representation)
export(requireMethods)
export(resetClass)
export(resetGeneric)
export(sealClass)
export(selectMethod)
export(seemsS4Object)
export(sessionData)
export(setAs)
export(setClass)
export(setClassUnion)
export(setDataPart)
export(setGeneric)
export(setGroupGeneric)
export(setIs)
export(setMethod)
export(setOldClass, .OldClassesList)
export(setPackageName)
export(setPrimitiveMethods)
export(setReplaceMethod)
export(setValidity)
export(show)
export(showClass)
export(showDefault)
export(showExtends)
export(showMethods)
export(showMlist)
export(sigToEnv)
export(signature)
export(slot, "slot<-", ".hasSlot")
export(slotNames, ".slotNames")
export(slotsFromS3)
export(substituteDirect)
export(substituteFunctionArgs)
export(superClassDepth)
export(testInheritedMethods)
export(testVirtual)
export(traceOff)
export(traceOn)
export(tryNew)
export(trySilent)
export(unRematchDefinition)
export(validObject)
export(validSlotNames)

## the next 2 would not be needed if R_initMethods looked in the namespace, not globalenv
export(.ShortPrimitiveSkeletons)
export(.EmptyPrimitiveSkeletons)

## to allow "S4-cbind/rbind (./R/cbind-rbind.R)
export(cbind2)
export(rbind2)

## implicit generic functionality

export(implicitGeneric)
export(setGenericImplicit)
export(prohibitGeneric)
export(registerImplicitGenerics)

## reference classes
export(setRefClass)
export(getRefClass)
export(initFieldArgs) # used in initialize methods
