import(methods)

import(graphics)
## instead of just
## importFrom("graphics", plot) # because we want to define methods on it
## *Still* do not want warning from this

## as "mgcv": this loads Matrix, but does not attach it  ==> Matrix methods "semi-visible"
importFrom(Matrix, t,mean,colMeans,colSums)

exportClasses("classA")
exportMethods("plot")
