The *___.afm files were downloaded from 
ftp://ftp.adobe.com/pub/adobe/type/win/all/afmfiles/base35
and coverted to LF line endings (from CRLF).
(Ths URI seems no longer available.)

The Courier*, Helvetica*, Times*, Symbol.afm and ZapfDingbats.afm files
were downloaded from 
http://partners.adobe.com/asn/tech/type/fontinfo/Core14_AFMs.zip 
and have licence file MustRead.html.  They are unmodified.

*l.afm are Copyright 1999 by (URW)++ Design & Development, distributed
under GPL as part of GhostScript 6.52 and copied from there.
Those for NimbusMon have been corrected to all be 'IsFixedPitch true'

CM_*.afm were converted by Brian D'Urso from afms which are copyright
by the American Mathematical Society, but `the AMS does require that
the AMS copyright notice be removed from any derivative versions of
the fonts which have been altered in any way'.


BDR 2001-01-13, 2001-11-28
