The scripts in this directory reproduce some of the examples from the book

  Snijders, Tom and Bosker, Roel (1999), Multilevel Analysis: An
    Introduction to Basic and Advanced Multilevel Modeling, Sage.
