The *___.afm files were downloaded from 
ftp://ftp.adobe.com/pub/adobe/type/win/all/afmfiles/base35
and coverted to LF line endings (from CRLF)

*l.afm are Copyright 1999 by (URW)++ Design & Development, distributed
under GPL as part of GhostScript 6.52 and copied from there.

CM_*.afm were converted by Brian D'Urso from afms which are copyright
by the American Mathematical Society, but `the AMS does require that
the AMS copyright notice be removed from any derivative versions of
the fonts which have been altered in any way'.


BDR 2001-01-13, 2001-11-28
