# Lattice

Lattice is a powerful and elegant high-level data visualization system
for R, inspired by Trellis graphics. It is designed with an emphasis
on multivariate data, and in particular allows easy conditioning to
produce "small multiple" plots. Lattice is sufficient for typical
graphics needs, and is also flexible enough to handle most nonstandard
requirements.

