These files are used by a program called aclocal (part of the GNU
automake package) which uses them to create aclocal.m4 which is in turn
used by autoconf to create the configure script at the the top level in
this distribution.

(Actually, as we need to change a few `standard' macros, we need to
create acinclude.m4 by concatenating all m4 files in this directory
before running aclocal.)

The files are as follows.

  R.m4
	contains R specific macros.  Some of these might be useful for
	other systems as well, but we first need to split this into
	pieces.
	R_BLAS_LIBS is based on the AC_ARG_WITH(fastblas) code from
	Octave, but with tests for Alpha CXML/DXML and IRIX SCSL and
	SGIMATH libraries commented out.
	Some macros do not conform to the Autoconf $PKG_$CATEGORY_$FOO
	naming scheme.  These test for `features' such as GNOME, zlib,
	etc., are named R_$FEATURE, and typically define HAVE_$FEATURE.

  codeset.m4
        is from GNU gettext and checks whether nl_langinfo(CODESET)
        is available in langinfo.h.

  gnome-gnorba-check.m4
  gnome-orbit-check.m4
  gnome.m4
  libglade.m4
	are from the GNOME project and used for the GNOME configuration.
	Changes made in gnome.m4:
	* Replace AC_ARG_WITH(gnome) by our version in configure.ac
	  because we need want_gnome=no by default and use LIBS and
	  CPPFLAGS rather than LDFLAGS and CFLAGS.
	* In AC_ARG_WITH(gnome-includes), standardize doc strings and
	  use CPPFLAGS rather than CFLAGS.
	* In AC_ARG_WITH(gnome-libs), standardize doc strings and use
	  LIBS rather than LDFLAGS.
	Changes made in libglade.m4:
	* In AC_ARG_WITH(libglade-config), standardize doc strings.

  libtool.m4
	is from libtool 1.5.
