The Fortran routines here are extracted from LAPACK 3.1.0, then 3.1.1, 3.4.1
(http://www.netlib.org/lapack and mirrors), then from SVN in July 2012.

dlapack?.f are support routines for `real' LAPACK calls
cmplx.f are support routines for LAPACK calls from complex arithmetic
routines, that are only used if COMPLEX*16 is supported by the Fortran
compiler and compatible with the Rcomplex type (guaranteed for f2c users).
Neither will not be compiled if an external LAPACK is used.

For 'base' R addition of a new .Call()able Lapack routine,
all these files must be updated {path relative to R's topdir}:

  src/gnuwin32/Rdll.hide
  src/include/R_ext/Lapack.h
  src/include/Rmodules/Rlapack.h
  src/main/basedecl.h
  src/main/lapack.c
  src/main/registration.c
  src/modules/lapack/Lapack.c
  src/modules/lapack/Lapack.h
  src/modules/lapack/cmplx.f or dlapack.f

We have tried to avoid including any single-precision routines.

There is also a patch to avoid a gfortran warning at lines 21131:2
of dlapack.f, in DGEQRT3.

dlapack.f is the concatenation of

dbbcsd.f	dgtsvx.f	dlarf.f		dorm2r.f	dstemr.f
dbdsdc.f	dgttrf.f	dlarfb.f	dormbr.f	dsteqr.f
dbdsqr.f	dgttrs.f	dlarfg.f	dormhr.f	dsterf.f
ddisna.f	dgtts2.f	dlarfgp.f	dorml2.f	dstev.f
dgbbrd.f	dhgeqz.f	dlarft.f	dormlq.f	dstevd.f
dgbcon.f	dhsein.f	dlarfx.f	dormql.f	dstevr.f
dgbequ.f	dhseqr.f	dlargv.f	dormqr.f	dstevx.f
dgbequb.f	disnan.f	dlarnv.f	dormr2.f	dsycon.f
dgbrfs.f	dlabad.f	dlarra.f	dormr3.f	dsyconv.f
dgbsv.f		dlabrd.f	dlarrb.f	dormrq.f	dsyequb.f
dgbsvx.f	dlacn2.f	dlarrc.f	dormrz.f	dsyev.f
dgbtf2.f	dlacon.f	dlarrd.f	dormtr.f	dsyevd.f
dgbtrf.f	dlacpy.f	dlarre.f	dpbcon.f	dsyevr.f
dgbtrs.f	dladiv.f	dlarrf.f	dpbequ.f	dsyevx.f
dgebak.f	dlae2.f		dlarrj.f	dpbrfs.f	dsygs2.f
dgebal.f	dlaebz.f	dlarrk.f	dpbstf.f	dsygst.f
dgebd2.f	dlaed0.f	dlarrr.f	dpbsv.f		dsygv.f
dgebrd.f	dlaed1.f	dlarrv.f	dpbsvx.f	dsygvd.f
dgecon.f	dlaed2.f	dlarscl2.f	dpbtf2.f	dsygvx.f
dgeequ.f	dlaed3.f	dlartg.f	dpbtrf.f	dsyrfs.f
dgeequb.f	dlaed4.f	dlartgp.f	dpbtrs.f	dsysv.f
dgees.f		dlaed5.f	dlartgs.f	dpftrf.f	dsysvx.f
dgeesx.f	dlaed6.f	dlartv.f	dpftri.f	dsyswapr.f
dgeev.f		dlaed7.f	dlaruv.f	dpftrs.f	dsytd2.f
dgeevx.f	dlaed8.f	dlarz.f		dpocon.f	dsytf2.f
dgegs.f		dlaed9.f	dlarzb.f	dpoequ.f	dsytrd.f
dgegv.f		dlaeda.f	dlarzt.f	dpoequb.f	dsytrf.f
dgehd2.f	dlaein.f	dlas2.f		dporfs.f	dsytri.f
dgehrd.f	dlaev2.f	dlascl.f	dposv.f		dsytri2.f
dgejsv.f	dlaexc.f	dlascl2.f	dposvx.f	dsytri2x.f
dgelq2.f	dlag2.f		dlasd0.f	dpotf2.f	dsytrs.f
dgelqf.f	dlags2.f	dlasd1.f	dpotrf.f	dsytrs2.f
dgels.f		dlagtf.f	dlasd2.f	dpotri.f	dtbcon.f
dgelsd.f	dlagtm.f	dlasd3.f	dpotrs.f	dtbrfs.f
dgelss.f	dlagts.f	dlasd4.f	dppcon.f	dtbtrs.f
dgelsx.f	dlagv2.f	dlasd5.f	dppequ.f	dtfsm.f
dgelsy.f	dlahqr.f	dlasd6.f	dpprfs.f	dtftri.f
dgemqrt.f	dlahr2.f	dlasd7.f	dppsv.f		dtfttp.f
dgeql2.f	dlahrd.f	dlasd8.f	dppsvx.f	dtfttr.f
dgeqlf.f	dlaic1.f	dlasda.f	dpptrf.f	dtgevc.f
dgeqp3.f	dlaisnan.f	dlasdq.f	dpptri.f	dtgex2.f
dgeqpf.f	dlaln2.f	dlasdt.f	dpptrs.f	dtgexc.f
dgeqr2.f	dlals0.f	dlaset.f	dpstf2.f	dtgsen.f
dgeqr2p.f	dlalsa.f	dlasq1.f	dpstrf.f	dtgsja.f
dgeqrf.f	dlalsd.f	dlasq2.f	dptcon.f	dtgsna.f
dgeqrfp.f	dlamrg.f	dlasq3.f	dpteqr.f	dtgsy2.f
dgeqrt.f	dlaneg.f	dlasq4.f	dptrfs.f	dtgsyl.f
dgeqrt2.f	dlangb.f	dlasq5.f	dptsv.f		dtpcon.f
dgeqrt3.f	dlange.f	dlasq6.f	dptsvx.f	dtpmqrt.f
dgerfs.f	dlangt.f	dlasr.f		dpttrf.f	dtpqrt.f
dgerq2.f	dlanhs.f	dlasrt.f	dpttrs.f	dtpqrt2.f
dgerqf.f	dlansb.f	dlassq.f	dptts2.f	dtprfb.f
dgesc2.f	dlansf.f	dlasv2.f	drscl.f		dtprfs.f
dgesdd.f	dlansp.f	dlaswp.f	dsbev.f		dtptri.f
dgesv.f		dlanst.f	dlasy2.f	dsbevd.f	dtptrs.f
dgesvd.f	dlansy.f	dlasyf.f	dsbevx.f	dtpttf.f
dgesvj.f	dlantb.f	dlatbs.f	dsbgst.f	dtpttr.f
dgesvx.f	dlantp.f	dlatdf.f	dsbgv.f		dtrcon.f
dgetc2.f	dlantr.f	dlatps.f	dsbgvd.f	dtrevc.f
dgetf2.f	dlanv2.f	dlatrd.f	dsbgvx.f	dtrexc.f
dgetrf.f	dlapll.f	dlatrs.f	dsbtrd.f	dtrrfs.f
dgetri.f	dlapmr.f	dlatrz.f	dsfrk.f		dtrsen.f
dgetrs.f	dlapmt.f	dlatzm.f	dspcon.f	dtrsna.f
dggbak.f	dlapy2.f	dlauu2.f	dspev.f		dtrsyl.f
dggbal.f	dlapy3.f	dlauum.f	dspevd.f	dtrti2.f
dgges.f		dlaqgb.f	dopgtr.f	dspevx.f	dtrtri.f
dggesx.f	dlaqge.f	dopmtr.f	dspgst.f	dtrtrs.f
dggev.f		dlaqp2.f	dorbdb.f	dspgv.f		dtrttf.f
dggevx.f	dlaqps.f	dorcsd.f	dspgvd.f	dtrttp.f
dggglm.f	dlaqr0.f	dorg2l.f	dspgvx.f	dtzrqf.f
dgghrd.f	dlaqr1.f	dorg2r.f	dsprfs.f	dtzrzf.f
dgglse.f	dlaqr2.f	dorgbr.f	dspsv.f		ieeeck.f
dggqrf.f	dlaqr3.f	dorghr.f	dspsvx.f	iladlc.f
dggrqf.f	dlaqr4.f	dorgl2.f	dsptrd.f	iladlr.f
dggsvd.f	dlaqr5.f	dorglq.f	dsptrf.f	ilaenv.f
dggsvp.f	dlaqsb.f	dorgql.f	dsptri.f	ilaprec.f
dgsvj0.f	dlaqsp.f	dorgqr.f	dsptrs.f	ilatrans.f
dgsvj1.f	dlaqsy.f	dorgr2.f	dstebz.f	iparmq.f
dgtcon.f	dlaqtr.f	dorgrq.f	dstedc.f
dgtrfs.f	dlar1v.f	dorgtr.f	dstegr.f
dgtsv.f		dlar2v.f	dorm2l.f	dstein.f

cmplx.f is the concatenation of

dzsum1.f	zgeqr2.f	zlahr2.f	zlarft.f	ztrtrs.f
ilazlc.f	zgeqrf.f	zlahrd.f	zlarfx.f	zung2l.f
ilazlr.f	zgesv.f		zlange.f	zlartg.f	zung2r.f
izmax1.f	zgesvd.f	zlanhe.f	zlascl.f	zungbr.f
zbdsqr.f	zgetf2.f	zlanhs.f	zlaset.f	zunghr.f
zdrscl.f	zgetrf.f	zlantr.f	zlasr.f		zungl2.f
zgebak.f	zgetrs.f	zlaqp2.f	zlassq.f	zunglq.f
zgebal.f	zheev.f		zlaqps.f	zlaswp.f	zungql.f
zgebd2.f	zhetd2.f	zlaqr0.f	zlatrd.f	zungqr.f
zgebrd.f	zhetrd.f	zlaqr1.f	zlatrs.f	zungr2.f
zgecon.f	zhseqr.f	zlaqr2.f	zpotf2.f	zungrq.f
zgeev.f		zlabrd.f	zlaqr3.f	zpotrf.f	zungtr.f
zgehd2.f	zlacgv.f	zlaqr4.f	zrot.f		zunm2r.f
zgehrd.f	zlacn2.f	zlaqr5.f	zsteqr.f	zunmbr.f
zgelq2.f	zlacpy.f	zlarf.f		ztrcon.f	zunml2.f
zgelqf.f	zladiv.f	zlarfb.f	ztrevc.f	zunmlq.f
zgeqp3.f	zlahqr.f	zlarfg.f	ztrexc.f	zunmqr.f
zunmhr.f

R.patch is a patch to suppress warnings.
