#  File src/library/tools/R/news.R
#  Part of the R package, http://www.R-project.org
#
#  Copyright (C) 2003 The R Core Team
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  http://www.r-project.org/Licenses/

{
    # NOTE: This file resides in the exec directory rather than the R
    # directory  because it does not define functions but is intended to
    # be sourced into a running R.
    library(tcltk)
    tkStartGUI()
}
