bibentry(bibtype = "Book",
         key = "R:Abramowitz+Stegun:1972",
         author = c(person(given = "Milton",
                           family = "Abramowitz"),
                    person(given = c("Irene", "A."),
                           family = "Stegun")),
         title = "Handbook of Mathematical Functions with Formulas, Graphs, and Mathematical Tables",
         publisher = "Dover Publications",
         year = "1972",
         address = "New York",
         isbn = "0486612724",
         pointers = "https://permalink.obvsg.at/AC00256630")

bibentry(bibtype = "Article",
         key = "R:Ahrens+Dieter:1973",
         author = c(person(given = c("Joachim", "H."),
                           family = "Ahrens"),
                    person(given = "Ulrich",
                           family = "Dieter")),
         title = "Extensions of {Forsythe}'s Method for Random Sampling from the Normal Distribution",
         year = "1973",
         journal = "Mathematics of Computation",
         volume = "27",
         pages = "927--937",
         publisher = "American Mathematical Society (AMS)",
         doi = "10.1090/S0025-5718-1973-0329190-8")

bibentry(bibtype = "Article",
         key = "R:Amos:1983",
         title = "{Algorithm 610}: A Portable {FORTRAN} Subroutine for Derivatives of the Psi Function",
         volume = "9",
         .issn = "1557-7295",
         doi = "10.1145/356056.356065",
         number = "4",
         journal = "ACM Transactions on Mathematical Software",
         publisher = "Association for Computing Machinery (ACM)",
         author = person(given = c("Donald", "E."),
                         family = "Amos"),
         year = "1983",
         month = "dec",
         pages = "494--502")

bibentry(bibtype = "Book",
         key = "R:Becker+Chambers+Wilks:1988",
         author = c(person(given = c("Richard", "A."),
                           family = "Becker"),
                    person(given = c("John", "M."),
                           family = "Chambers"),
                    person(given = c("Allan", "R."),
                           family = "Wilks")),
         title = "The New {S} Language",
         publisher = "Chapman and Hall/CRC",
         year = "1988",
         address = "London",
         isbn = "053409192X",
         pointers = "https://permalink.obvsg.at/AC00095416")

bibentry(bibtype = "Article",
         key = "R:Box+Muller:1958",
         title = "A Note on the Generation of Random Normal Deviates",
         volume = "29",
         .issn = "0003-4851",
         doi = "10.1214/aoms/1177706645",
         number = "2",
         journal = "The Annals of Mathematical Statistics",
         publisher = "Institute of Mathematical Statistics",
         author = c(person(given = c("George", "Edward", "Pelham"),
                           family = "Box"),
                    person(given = c("Mervin", "Edgar"),
                           family = "Muller")),
         year = "1958",
         month = "jun",
         pages = "610--611")

bibentry(bibtype = "Article",
         key = "R:Campbell:1979",
         title = "{Bessel} Functions $J_\\nu(x)$ and $Y_\\nu(x)$ of Real Order and Real Argument",
         volume = "18",
         .issn = "0010-4655",
         doi = "10.1016/0010-4655(79)90030-4",
         number = "1",
         journal = "Computer Physics Communications",
         publisher = "Elsevier BV",
         author = person(given = c("J.", "B."),
                         family = "Campbell"),
         year = "1979",
         month = "sep",
         pages = "133--142")

bibentry(bibtype = "Article",
         key = "R:Campbell:1980",
         title = "On {Temme}’s Algorithm for the Modified {Bessel} Function of the Third Kind",
         volume = "6",
         .issn = "1557-7295",
         doi = "10.1145/355921.355928",
         number = "4",
         journal = "ACM Transactions on Mathematical Software",
         publisher = "Association for Computing Machinery (ACM)",
         author = person(given = c("J.", "B."),
                         family = "Campbell"),
         year = "1980",
         month = "dec",
         pages = "581--586")

bibentry(bibtype = "InCollection",
         key = "R:Chambers:1992:c3",
         author = person(given = c("John", "M."),
                           family = "Chambers"),
         title = "Data for Models",
         booktitle = "Statistical Models in {S}",
         publisher = "Wadsworth & Brooks/Cole",
         year = "1992",
         editor = c(person(given = c("John", "M."),
                           family = "Chambers"),
                    person(given = c("Trevor", "J."),
                           family = "Hastie")),
         chapter = "3")

bibentry(bibtype = "InCollection",
         key = "R:Chambers:1992:c4",
         author = person(given = c("John", "M."),
                           family = "Chambers"),
         title = "Linear Models",
         booktitle = "Statistical Models in {S}",
         publisher = "Wadsworth & Brooks/Cole",
         year = "1992",
         editor = c(person(given = c("John", "M."),
                           family = "Chambers"),
                    person(given = c("Trevor", "J."),
                           family = "Hastie")),
         chapter = "4")

bibentry(bibtype = "InCollection",
         key = "R:Chambers:1992:aa",
         author = person(given = c("John", "M."),
                           family = "Chambers"),
         title = "Classes and Methods: Object-Oriented Programming in {S}",
         booktitle = "Statistical Models in {S}",
         publisher = "Wadsworth & Brooks/Cole",
         year = "1992",
         editor = c(person(given = c("John", "M."),
                           family = "Chambers"),
                    person(given = c("Trevor", "J."),
                           family = "Hastie")),
         note = "Appendix A")

bibentry(bibtype = "Book",
         key = "R:Chambers+Cleveland+Kleiner:1983",
         title = "Graphical Methods for Data Analysis",
         author = c(person(given = c("John", "M."),
                           family = "Chambers"),
                    person(given = c("William", "S."),
                           family = "Cleveland"),
                    person(given = "Beat",
                           family = "Kleiner"),
                    person(given = c("Paul", "A."),
                           family = "Tukey")),
         year = "1983",
         publisher = "Wadsworth & Brooks/Cole",
         isbn = "0871504138",
         pointers = "https://permalink.obvsg.at/AC02269805")

bibentry(bibtype = "Book",
         key = "R:Chambers+Hastie:1992",
         author = c(person(given = c("John", "M."),
                           family = "Chambers"),
                    person(given = c("Trevor", "J."),
                           family = "Hastie")),
         title = "Statistical Models in {S}",
         publisher = "Chapman & Hall",
         year = "1992",
         address = "London",
         isbn = "9780412830402",
         publisherurl = "http://www.crcpress.com/product/isbn/9780412830402")

bibentry(bibtype = "InCollection",
         key = "R:Chambers+Hastie:1992:c2",
         author = c(person(given = c("John", "M."),
                           family = "Chambers"),
                    person(given = c("Trevor", "J."),
                           family = "Hastie")),
         title = "Statistical Models",
         booktitle = "Statistical Models in {S}",
         publisher = "Wadsworth & Brooks/Cole",
         year = "1992",
         editor = c(person(given = c("John", "M."),
                           family = "Chambers"),
                    person(given = c("Trevor", "J."),
                           family = "Hastie")),
         chapter = "2")

bibentry(bibtype = "Book",
         key = "R:Chambers:1998",
         author = person(given = c("John", "M."),
                         family = "Chambers"),
         title = "Programming with Data",
         publisher = "Springer",
         year = "1998",
         address = "New York",
         isbn = "978-0-387-98503-9",
         publisherurl = "https://www.springer.com/978-0-387-98503-9")

bibentry(bibtype = "Book",
         key = "R:Chambers:2008",
         author = person(given = c("John", "M."),
                         family = "Chambers"),
         title = "Software for Data Analysis: Programming with {R}",
         publisher = "Springer",
         year = "2008",
         address = "New York",
         isbn = "978-0-387-75935-7",
         url = "https://johnmchambers.su.domains/Rbook/",
         publisherurl = "https://www.springer.com/book/978-0-387-75935-7")

bibentry(bibtype = "Article",
         key = "R:Cody:1983",
         title = "{Algorithm 597}: Sequence of Modified {Bessel} Functions of the First Kind",
         volume = "9",
         .issn = "1557-7295",
         doi = "10.1145/357456.357462",
         number = "2",
         journal = "ACM Transactions on Mathematical Software",
         publisher = "Association for Computing Machinery (ACM)",
         author = person(given = c("William", "James"),
                         family = "Cody"),
         year = "1983",
         month = "jun",
         pages = "242--245")

bibentry(bibtype = "Article",
         key = "R:Cody:1988",
         title = "{Algorithm 665}: {Machar}: A Subroutine to Dynamically Determined Machine Parameters",
         volume = "14",
         .issn = "1557-7295",
         doi = "10.1145/50063.51907",
         number = "4",
         journal = "ACM Transactions on Mathematical Software",
         publisher = "Association for Computing Machinery (ACM)",
         author = person(given = c("William", "James"),
                         family = "Cody"),
         year = "1988",
         month = "dec",
         pages = "303--311")

bibentry(bibtype = "Article",
         key = "R:De_Matteis+Pagnutti:1993",
         title = "Long-range Correlation Analysis of the {Wichmann}-{Hill} Random Number Generator",
         volume = "3",
         .issn = "1573-1375",
         doi = "10.1007/BF00153065",
         number = "2",
         journal = "Statistics and Computing",
         publisher = "Springer Science and Business Media LLC",
         author = c(person(given = "A.",
                           family = "De Matteis"),
                    person(given = "Simonetta",
                           family = "Pagnutti")),
         year = "1993",
         month = "jun",
         pages = "67--70",
         .doi = "10.1007/bf00153065")

bibentry(bibtype = "Article",
         key = "R:Goldberg:1991",
         title = "What Every Computer Scientist Should Know about Floating-point Arithmetic",
         volume = "23",
         .issn = "1557-7341",
         doi = "10.1145/103162.103163",
         number = "1",
         journal = "ACM Computing Surveys",
         publisher = "Association for Computing Machinery (ACM)",
         author = person(given = "David",
                         family = "Goldberg"),
         year = "1991",
         month = "mar",
         pages = "5--48")

bibentry(bibtype = "Book",
         key = "R:Kernighan+Ritchie:1988",
         title = "The {C} Programming Language",
         author = c(person(given = c("Brian", "W."),
                           family = "Kernighan"),
                    person(given = c("Dennis", "M."),
                           family = "Ritchie")),
         isbn = "9789688802052",
         lccn = "77028983",
         series = "Prentice-Hall software series",
         year = "1988",
         publisher = "Prentice Hall")

bibentry(bibtype = "Article",
         key = "R:Kinderman+Ramage:1976",
         title = "Computer Generation of Normal Random Variables",
         volume = "71",
         .issn = "1537-274X",
         doi = "10.1080/01621459.1976.10480965",
         number = "356",
         journal = "Journal of the American Statistical Association",
         publisher = "Informa UK Limited",
         author = c(person(given = c("Albert", "J."),
                           family = "Kinderman"),
                    person(given = c("J.", "G."),
                           family = "Ramage")),
         year = "1976",
         month = "dec",
         pages = "893--896",
         .doi = "10.2307/2286857")

bibentry(bibtype = "Book",
         key = "R:Knuth:1972:v1",
         title = "The Art of Computer Programming, Volume 1: Fundamental Algorithms",
         author = person(given = c("Donald", "Ervin"),
                         family = "Knuth",
                         comment = c(ORCID = "0009-0000-8366-9524")),
         year = "1972",
         isbn = "978-0201038095",
         publisher = "Addison-Wesley")

bibentry(bibtype = "Book",
         key = "R:Knuth:1997:v2",
         title = "The Art of Computer Programming, Volume 2: Seminumerical Algorithms",
         author = person(given = c("Donald", "Ervin"),
                         family = "Knuth",
                         comment = c(ORCID = "0009-0000-8366-9524")),
         year = "1997",
         edition = "3rd",
         publisher = "Addison-Wesley Professional",
         isbn = "0201896842",
         pointers = "https://permalink.obvsg.at/AC02114194")

bibentry(bibtype = "Book",
         key = "R:Knuth:1998:v3",
         title = "The Art of Computer Programming, Volume 3: Sorting and Searching",
         author = person(given = c("Donald", "Ervin"),
                         family = "Knuth",
                         comment = c(ORCID = "0009-0000-8366-9524")),
         year = "1998",
         edition = "2nd",
         publisher = "Addison-Wesley Professional",
         isbn = "0201896850",
         pointers = "https://permalink.obvsg.at/AC02410169")

bibentry(bibtype = "Book",
         key = "R:Knuth:2002:v3",
         title = "The Art of Computer Programming, Volume 2: Seminumerical Algorithms",
         author = person(given = c("Donald", "Ervin"),
                         family = "Knuth",
                         comment = c(ORCID = "0009-0000-8366-9524")),
         year = "2002",
         edition = "3rd",
         publisher = "Addison-Wesley Professional",
         note = "Ninth printing",
         isbn = "0201896842",
         pointers = "https://permalink.obvsg.at/AC03569894")

bibentry(bibtype = "Article",
         key = "R:L_Ecuyer+Simard:2007",
         title = "{TestU01}: A {C} Library for Empirical Testing of Random Number Generators",
         volume = "33",
         .issn = "1557-7295",
         doi = "10.1145/1268776.1268777",
         number = "4",
         journal = "ACM Transactions on Mathematical Software",
         publisher = "Association for Computing Machinery (ACM)",
         author = c(person(given = "Pierre",
                           family = "L'Ecuyer",
                           comment = c(ORCID = "0000-0002-3184-0796")),
                    person(given = "Richard",
                           family = "Simard")),
         year = "2007",
         month = "aug",
         pages = "1--40")

bibentry(bibtype = "Article",
         key = "R:L_Ecuyer:1999",
         title = "Good Parameters and Implementations for Combined Multiple Recursive Random Number Generators",
         volume = "47",
         .issn = "1526-5463",
         doi = "10.1287/opre.47.1.159",
         number = "1",
         journal = "Operations Research",
         publisher = "Institute for Operations Research and the Management Sciences (INFORMS)",
         author = person(given = "Pierre",
                         family = "L'Ecuyer",
                         comment = c(ORCID = "0000-0002-3184-0796")),
         year = "1999",
         month = "feb",
         pages = "159--164")

bibentry(bibtype = "Misc",
         key = "R:Lumley:2003",
         author = person(given = "Thomas",
                         family = "Lumley"),
         title = "Standard Nonstandard Evaluation Rules",
         year = "2003",
         url = "https://developer.r-project.org/nonstandard-eval.pdf")

bibentry(bibtype = "Article",
         key = "R:Marsaglia+Zaman:1994",
         title = "Some Portable Very-long-period Random Number Generators",
         volume = "8",
         .issn = "0894-1866",
         doi = "10.1063/1.168514",
         number = "1",
         journal = "Computers in Physics",
         publisher = "AIP Publishing",
         author = c(person(given = "George",
                           family = "Marsaglia"),
                    person(given = "Arif",
                           family = "Zaman")),
         year = "1994",
         month = "jan",
         pages = "117--121")

bibentry(bibtype = "Misc",
         key = "R:Marsaglia:1997",
         author = person(given = "George",
                         family = "Marsaglia"),
         title = "A Random Number Generator for {C}",
         year = "1997",
         note = "Discussion paper, posting on Usenet newsgroup \\code{sci.stat.math} on September 29, 1997.")

bibentry(bibtype = "Article",
         key = "R:Matsumoto+Nishimura:1998",
         author = c(person(given = "Makoto",
                           family = "Matsumoto"),
                    person(given = "Takuji",
                           family = "Nishimura")),
         title = "Mersenne Twister: A 623-dimensionally Equidistributed Uniform Pseudo-random Number Generator",
         year = "1998",
         publisher = "Association for Computing Machinery",
         address = "New York, NY, USA",
         volume = "8",
         number = "1",
         .issn = "1049-3301",
         doi = "10.1145/272991.272995",
         journal = "ACM Transactions on Modeling and Computer Simulation",
         month = "jan",
         pages = "3--30",
         numpages = "28")

bibentry(bibtype = "Article",
         key = "R:Murdoch:2010",
         title = "Source References",
         volume = "2",
         .issn = "2073-4859",
         doi = "10.32614/RJ-2010-010",
         number = "2",
         journal = "The R Journal",
         publisher = "The R Foundation",
         author = person(given = "Duncan",
                         family = "Murdoch"),
         year = "2010",
         pages = "16")

bibentry(bibtype = "Misc",
         key = "R:Reeds+Hubert+Abrahams:1982-4",
         title = "{C} implementation of {SuperDuper}",
         author = c(person("Jim", "Reeds"),
                    person("S.", "Hubert"),
                    person("M.", "Abrahams")),
         year = "1982--4",
         note = "University of California at Berkeley.  (Personal communication from Jim Reeds to Ross Ihaka.)")

bibentry(bibtype = "Article",
         key = "R:Ripley:2001",
         author = person(given = c("Brian", "D."),
                         family = "Ripley"),
         title = "Connections",
         journal = "R News",
         year = "2001",
         volume = "1",
         number = "1",
         pages = "16--17",
         month = "January",
         url = "https://journal.r-project.org/articles/RN-2001-005/")

bibentry(bibtype = "Article",
         key = "R:Ripley+Hornik:2001",
         author = c(person(given = c("Brian", "D."),
                           family = "Ripley"),
                    person(given = "Kurt",
                           family = "Hornik",
                           comment = c(ORCID = "0000-0003-4198-9911"))),
         title = "Date-Time Classes",
         journal = "R News",
         year = "2001",
         volume = "1",
         number = "2",
         pages = "8--11",
         month = "June",
         url = "https://journal.r-project.org/articles/RN-2001-011/")

bibentry(bibtype = "Article",
         key = "R:Sedgewick:1986",
         title = "A New Upper Bound for {Shellsort}",
         volume = "7",
         .issn = "0196-6774",
         doi = "10.1016/0196-6774(86)90001-5",
         number = "2",
         journal = "Journal of Algorithms",
         publisher = "Elsevier BV",
         author = person(given = "Robert",
                         family = "Sedgewick"),
         year = "1986",
         month = "jun",
         pages = "159--173")

bibentry(bibtype = "Article",
         key = "R:Singleton:1969",
         title = "{Algorithm 347}: An Efficient Algorithm for Sorting with Minimal Storage [M1]",
         volume = "12",
         .issn = "1557-7317",
         doi = "10.1145/362875.362901",
         number = "3",
         journal = "Communications of the ACM",
         publisher = "Association for Computing Machinery (ACM)",
         author = person(given = c("Richard", "C."),
                         family = "Singleton"),
         year = "1969",
         month = "mar",
         pages = "185--187")

bibentry(bibtype = "Article",
         key = "R:Sookne:1973",
         title = "{Bessel} Functions $I$ and $J$ of Complex Argument and Integer Order",
         volume = "77B",
         .issn = "0098-8979",
         doi = "10.6028/jres.077B.012",
         number = "3 and 4",
         journal = "Journal of Research of the National Bureau of Standards, Section B: Mathematical Sciences",
         publisher = "National Institute of Standards and Technology (NIST)",
         author = person(given = c("David", "J."),
                         family = "Sookne"),
         year = "1973",
         month = "jul",
         pages = "111")

bibentry(bibtype = "Article",
         key = "R:Temme:1976",
         title = "On the Numerical Evaluation of the Ordinary {Bessel} Function of the Second Kind",
         volume = "21",
         .issn = "0021-9991",
         doi = "10.1016/0021-9991(76)90032-2",
         number = "3",
         journal = "Journal of Computational Physics",
         publisher = "Elsevier BV",
         author = person(given = c("Nico", "M."),
                         family = "Temme"),
         year = "1976",
         month = "jul",
         pages = "343--350")

bibentry(bibtype = "Article",
         key = "R:Temple_Lang:2001a",
         author = person(given = "Duncan",
                         family = c("Temple Lang"),
                         comment = c(ORCID = "0000-0003-0159-1546")),
         title = "In Search of {C/C++} & {FORTRAN} Routines",
         journal = "R News",
         year = "2001",
         volume = "1",
         number = "3",
         pages = "20--23",
         month = "September",
         url = "https://journal.r-project.org/articles/RN-2001-024/")

bibentry(bibtype = "Article",
         key = "R:Jenkins+Traub:1972",
         title = "{Algorithm 419}: Zeros of a Complex Polynomial [C2]",
         volume = "15",
         .issn = "1557-7317",
         doi = "10.1145/361254.361262",
         number = "2",
         journal = "Communications of the ACM",
         publisher = "Association for Computing Machinery (ACM)",
         author = c(person(given = c("Michael", "A."),
                           family = "Jenkins"),
                    person(given = c("Joseph", "F."),
                           family = "Traub")),
         year = "1972",
         month = "feb",
         pages = "97--99")

bibentry(bibtype = "Article",
         key = "R:Wichmann+Hill:1982",
         title = "{Algorithm AS 183}: An Efficient and Portable Pseudo-random Number Generator",
         volume = "31",
         .issn = "0035-9254",
         doi = "10.2307/2347988",
         number = "2",
         journal = "Applied Statistics",
         publisher = "JSTOR",
         author = c(person(given = c("Brian", "A."),
                           family = "Wichmann"),
                    person(given = c("I.", "David"),
                           family = "Hill")),
         year = "1982",
         pages = "188--190")

bibentry(bibtype = "Article",
         key = "R:Wichmann+Hill:1984",
         title = "Correction: {Algorithm AS 183}: An Efficient and Portable Pseudo-Random Number Generator",
         volume = "33",
         .issn = "0035-9254",
         doi = "10.2307/2347676",
         number = "1",
         journal = "Applied Statistics",
         publisher = "JSTOR",
         author = c(person(given = c("Brian", "A."),
                           family = "Wichmann"),
                    person(given = c("I.", "David"),
                           family = "Hill")),
         year = "1984",
         pages = "123")

bibentry(bibtype = "Book",
         key = "R:Anderson+Bai+Bischof:1999",
         title = "{LAPACK} Users' Guide",
         author = c(person(given = "Edward",
                           family = "Anderson"),
                    person(given = "Zhengwu",
                           family = "Bai"),
                    person(given = "Christian",
                           family = "Bischof"),
                    person(given = "Susan",
                           family = "Blackford"),
                    person(given = "James",
                           family = "Demmel"),
                    person(given = c("Jack", "Joseph"),
                           family = "Dongarra"),
                    person(given = "Jeremy",
                           family = "Du Croz"),
                    person(given = "Anne",
                           family = "Greenbaum"),
                    person(given = "Sven",
                           family = "Hammerling"),
                    person(given = "A.",
                           family = "McKenney"),
                    person(given = c("Danny", "C."),
                           family = "Sorensen")),
         isbn = "9780898714470",
         lccn = "99048954",
         series = "Software, Environments, and Tools",
         edition = "Third",
         year = "1999",
         publisher = "Society for Industrial and Applied Mathematics",
         address = "Philadelphia, PA",
         doi = "10.1137/1.9780898719604",
         url = "https://netlib.org/lapack/lug/lapack_lug.html")

bibentry(bibtype = "Book",
         key = "R:Dongarra+Bunch+Moler:1979",
         title = "{LINPACK} Users' Guide",
         author = c(person(given = c("Jack", "Joseph"),
                           family = "Dongarra"),
                    person(given = c("James", "R."),
                           family = "Bunch"),
                    person(given = c("Cleve", "Barry"),
                           family = "Moler"),
                    person(given = c("Gilbert", "W."),
                           family = "Stewart")),
         isbn = "9780898711721",
         lccn = "78078206",
         series = "Other Titles in Applied Mathematics",
         year = "1979",
         publisher = "Society for Industrial and Applied Mathematics",
         doi = "10.1137/1.9781611971811")

bibentry(bibtype = "Misc",
         key = "R:Temple_Lang:2001b",
         author = person(given = "Duncan",
                         family = "Temple Lang",
                         comment = c(ORCID = "0000-0003-0159-1546")),
         title = "Top-level Task Callbacks in {R}",
         year = "2001",
         url = "https://developer.r-project.org/TaskHandlers.pdf")

bibentry(bibtype = "Book",
         key = "R:Wilkinson:1965",
         author = person(given = c("James", "H."),
                         family = "Wilkinson"),
         title = "The Algebraic Eigenvalue Problem",
         publisher = "Clarendon Press",
         year = "1965",
         address = "Oxford",
         isbn = "0198534035",
         pointers = "https://permalink.obvsg.at/AC00909584")

bibentry(bibtype = "Book",
         key = "R:Ripley:1987",
         author = person(given = c("Brian", "D."),
                         family = "Ripley"),
         title = "Stochastic Simulation",
         publisher = "Wiley",
         year = "1987",
         address = "New York, NY",
         isbn = "0471818844",
         pointers = "https://permalink.obvsg.at/AC00156144")

bibentry(bibtype = "Book",
         key = "R:Searle:1982",
         author = person(given = c("Shayle", "R."),
                         family = "Searle"),
         title = "Matrix Algebra Useful for Statistics",
         publisher = "Wiley",
         year = "1982",
         address = "New York",
         isbn = "0471866814",
         pointers = "https://permalink.obvsg.at/AC00639690")

bibentry(bibtype = "Book",
         key = "R:Powers+Peek+O_Reilly:2002",
         author = c(person(given = "Shelley",
                           family = "Powers"),
                    person(given = "Jerry",
                           family = "Peek"),
                    person(given = "Tim",
                           family = "O'Reilly"),
                    person(given = "Mike",
                           family = "Loukides")),
         title = "Unix Power Tools",
         publisher = "O'Reilly Media, Inc.",
         year = "2002",
         edition = "Third",
         isbn = "0596003307",
         pointers = "https://www.oreilly.com/library/view/unix-power-tools/0596003307/")

bibentry(bibtype = "Manual",
         key = "R:International_Organization_for_Standardization:2000",
         title = "Data Elements and Interchange Formats --- Information Interchange --- Representation of Dates and Times",
         author = person(given = "International Organization for Standardization"),
         organization = "International Organization for Standardization",
         address = "Geneva, Switzerland",
         year = "2000",
         edition = "ISO 8601:2000",
         url = "https://www.iso.org/standard/26780.html")

bibentry(bibtype = "Manual",
         key = "R:International_Organization_for_Standardization:2004",
         title = "Data Elements and Interchange Formats --- Information Interchange --- Representation of Dates and Times",
         author = person(given = "International Organization for Standardization"),
         organization = "International Organization for Standardization",
         address = "Geneva, Switzerland",
         year = "2004",
         edition = "ISO 8601:2004",
         url = "https://www.iso.org/standard/40874.html")

bibentry(bibtype = "Manual",
         key = "R:International_Organization_for_Standardization:2019",
         title = "Date and Time --- Representations for Information Interchange Part 1: Basic Rules",
         author = person(given = "International Organization for Standardization"),
         organization = "International Organization for Standardization",
         address = "Geneva, Switzerland",
         edition = "ISO 8601-1:2019",
         year = "2019",
         url = "https://www.iso.org/standard/70907.html")

bibentry(bibtype = "Manual",
         key = "R:International_Organization_for_Standardization:2022",
         title = "Date and Time --- Representations for Information Interchange Part 1: Basic Rules Amendment 1: Technical Corrections",
         author = person(given = "International Organization for Standardization"),
         organization = "International Organization for Standardization",
         address = "Geneva, Switzerland",
         edition = "ISO 8601-1:2019/Amd 1",
         year = "2022",
         url = "https://www.iso.org/standard/81801.html")
