#  File src/library/stats/R/ave.R
#  Part of the R package, http://www.R-project.org
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  http://www.r-project.org/Licenses/

ave <- function (x, ..., FUN = mean)
{
    n <- length(list(...))
    if (n) {
	g <- interaction(...)
	split(x,g) <- lapply(split(x, g), FUN)
    } else
        x[] <- FUN(x)
    x
}
