sum <- function(..., na.rm = FALSE)
    .Internal(sum(..., na.rm = na.rm))

min <- function(..., na.rm = FALSE)
    .Internal(min(..., na.rm = na.rm))

max <- function(..., na.rm = FALSE)
    .Internal(max(..., na.rm = na.rm))

prod <- function(..., na.rm = FALSE)
    .Internal(prod(..., na.rm = na.rm))

all <- function(..., na.rm = FALSE)
    .Internal(all(..., na.rm = na.rm))

any <- function(..., na.rm = FALSE)
    .Internal(any(..., na.rm = na.rm))
