%
% NOTE -- ONLY EDIT THE .Rnw FILE!!!  The .tex file is
% likely to be overwritten.
%
%\VignetteIndexEntry{Bioconductor Overview}
%\VignetteDepends{}
%\VignetteKeywords{Documentation}
%\VignettePackage{Biobase}
\documentclass[12pt]{article}

\usepackage{times}
\usepackage{hyperref}


\textwidth=6.2in
\textheight=8.5in
%\parskip=.3cm
\oddsidemargin=.1in
\evensidemargin=.1in
\headheight=-.3in

\newcommand{\scscst}{\scriptscriptstyle}
\newcommand{\scst}{\scriptstyle}

\bibliographystyle{plainnat}

\begin{document}

\title{What is Bioconductor}
\maketitle

\section*{Prequisites}

Bioconductor is a project to develop innovative software tools for use
in computational biology.
It is based on the R language ({\url www.r-project.org}).
You should already be quite familiar with R before using
Bioconductor. There are several on--line resources that can help you
get started using R. They can be found from the R website.
Some users find this a very steep learning curve; your experience may
be similar.

Bioconductor packages provide flexible interactive tools for carrying
out a number of different computational tasks. They are generally not
as fast as other analysis tools (since they are interactive) and often
reflect current ideas. Most can be improved and interested users should 
file bug reports and feature requests on the Bioconductor mailing list.


Bioconductor welcome collaboration in many different forms. These
include new packages, fixes or additions to existing packages and help
on different projects that are currently underway.
Please see the {\em Current Projects} web page to see if there are any
projects that are intersting to you.

\subsection*{How to report a bug}

Please provide enough information for us to help you. This typically
includes the platform (windows, Unix, Macintosh) that you are using as
well as version numbers for R and for the package that seems to be
working incorrectly.

Include a small complete example that can be run and demonstrates the
problem. In some cases it is also important that you describe what you
thought you should get.

Please note:
\begin{itemize}
\item bugs in R should be reported to the R community
\item missing features are not bugs -- they are feature requests.
\end{itemize}

\section{Bioconductor Design}

Bioconductor relies on the R package system to distribute code and
data. Most packages use S4 methods and classes (as described in {\em
  Programming with Data} by J. M. Chambers). This adherence to object
oriented programming makes it easier to build component software and
helps to deal with the complexity of the data.

One of the important Bioconductor packages is {\em annotate}. This
package provides access to various genomic annotation data. 
This package makes use of various web resources and precompiled data
packages to provide tools for exploring biological data.

\section{Session Information}

The version number of R and packages loaded for generating the vignette were:

\begin{verbatim}
<<echo=FALSE,results=tex>>=
sessionInfo()
@
\end{verbatim}

\end{document}
