\documentclass{article}
\SweaveOpts{engine=R,eps=FALSE,pdf=TRUE,strip.white=true,keep.source=TRUE}

\usepackage{Sweave}

\begin{document}
<<preliminaries, echo=FALSE, results=hide>>=
options(width=70, useFancyQuotes = FALSE, prompt="R> ", continue="+  ")
@
\subsection*{Introduction}

We generate 3D gaussian data,
<<ex1-U3>>=
set.seed(1)
n <- 100
x <- rnorm(n); y <- 2*x + rnorm(n)/2
U3 <- cbind(x, y, z = -3*x + y + rnorm(n)/4)
@
look at its structure
<<str>>=
str(U3) # its structure ((comment kept))
@
and load package \texttt{lattice}
<<req-lattice>>=
require("lattice")
@
to visualize it by a simple scatter plot matrix
\begin{figure}[h!]
\centering
<<splom-def, eval=false>>=
splom(U3, xlab ="", cex = 0.4)
<<splom, echo=FALSE, fig=TRUE, height=5>>=
print(
<<splom-def>>
     )
@
\caption{\Sexpr{n} vectors of random variates ... ...}
\label{fig:AC_Joe}
\end{figure}

\subsection*{Session Information}

<<sessionInfo, results=tex>>=
toLatex(sessionInfo())
@

\end{document}
