\name{package.manager}
\alias{package.manager}
\title{Package Manager}
\description{
  Graphical interface for managing packages currently installed on the system. 
}
\usage{
package.manager()
}
}
\details{
  This interface shows the current state of packages present on your 
  system. A graphical browser shows which packages are currently
  loaded in RAqua. The user can select packages to be loaded or detached
  upon exiting this browser.
}
\seealso{
  \code{\link{library}}, \code{\link{detach}}.
}
}
\keyword{documentation}
