\name{data.manager}
\alias{data.manager}
\title{Data Manager}
\description{
  Graphical interface for managing datasets currently installed on the system. 
}
\usage{
data.manager()
}
}
\details{
  This interface shows datasets distributed with packages that are currently
  loaded into RAqua. A graphical browser shows dataset name and package.
  The user can select datasets to be loaded upon exiting this browser.
}
\seealso{
  \code{\link{data}}.
}
}
\keyword{documentation}
