\name{citation}
\title{Citing R in Publications}
\alias{citation}
\description{
  How to cite R in publications.
}
\usage{
citation()
}
\details{
  Execute function \code{citation()} for information on how to cite R in
  publications. 
}
\keyword{misc}
