% File src/library/datasets/man/lynx.Rd
% Part of the R package, https://www.R-project.org
% Copyright 1995-2018 R Core Team
% Distributed under GPL 2 or later

\name{lynx}
\docType{data}
\alias{lynx}
\title{Annual Canadian Lynx trappings 1821--1934}
\description{
  Annual numbers of lynx trappings for 1821--1934 in Canada. Taken from
  \bibcite{Brockwell & Davis (1991)}, this appears to be the series considered
  by \bibcite{Campbell & Walker (1977)}.
}
\usage{lynx}
\source{
  Brockwell, P. J. and Davis, R. A. (1991).
  \emph{Time Series and Forecasting Methods}.  Second edition.
  Springer. Series G (page 557).
}
\references{
  Becker, R. A., Chambers, J. M. and Wilks, A. R. (1988).
  \emph{The New S Language}.
  Wadsworth & Brooks/Cole.

  Campbell, M. J. and Walker, A. M. (1977).
  A Survey of statistical work on the Mackenzie River series of annual
  Canadian lynx trappings for the years 1821--1934 and  a new analysis.
  \emph{Journal of the Royal Statistical Society Series A}, \bold{140},
  411--431.
  \doi{10.2307/2345277}.
}
\keyword{datasets}
